/*
 * Created on Aug 17, 2003
 * 
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package VM2;

/*--------------------------------------------------------------------
 * Generic Camera Interface
 * This interface is available in a PC and a jStik version
 * In the PC version is simulates the availability of a
 * local camera by reading data from the screen buffer
 * On the jStik a direct and a simulated interface are
 * available. The direct interface reads from the capture
 * buffer, the simulator is only used for timing experiment
 * Please note that the X and Y settings refer to:
 * - X is the line number
 * - Y is the column number
 *-------------------------------------------------------------------
 */

public class CameraHandler {
	public static final byte 
		LOG_5X5_A = 1, 
		LOG_5X5_B = 2, 
		FIRST_X_3X3 = 3,
		FIRST_Y_3X3 = 4, 
		LAP_3X3 = 5, 
		LAP_5X5 = 6, 
		GAUSS_3X3 = 7,
		GAUSS_5X1 = 8, 
		GAUSS_1X5 = 9, 
		GAUSS_5X1_X = 10, 
		GAUSS_1X5_Y = 11,
		GAUSS_5X5_XY = 12, 
		GAUSS_5X1_XX = 13, 
		GAUSS_1X5_YY = 14,
		GAUSS_5X5 = 15, 
		POS_ALL = 16,
		STAMP_5X5 = 17,
		STAMP2_5X5 = 18;

	public static final byte 
		BUFFERID_IN = 1, 
		BUFFERID_OUT = 0,
		BUFFERID_CUSTOM1 = 2,
		BUFFERID_BACKGR = 3;

	private Camera camera;

	private byte[] bufferOut; // This buffer will be the final output
	private byte[] bufferIn; // New frames get fed into this buffer
	private byte[] bufferCustom1; // An extra buffer for custom use
	private byte[] bufferBackground;
	
	private byte[] bufferSource; // A reference to the buffer intended for input
	private byte[] bufferScratch; // A reference to the buffer intended for in
								  // between results
	private byte[] bufferDestination; // A reference to the buffer intended for
									  // output

	private int bufInx = 0;
	private int bufLine = 0;
	private int bufPos = 0;
	private double mouseX = 0;
	private double mouseY = 0;

	private byte[][] convolutionMatrix; // the set convolution matrix, for
										// convolve()
	private byte normalizeFactor; // factor to divide with after convolution
	private boolean matrixNegative; // indicates whether matrix contains
									// negative values

	private int[] lastPixel = new int[3]; // last result of convolve() (perhaps
										  // other methods later as well)

	public CameraHandler(String socket) {
		if (socket.equals("USB")) {
			camera = new USBCam();
		} else if (socket.equals("Local")) {
			camera = new LocalCam();
		} else {
			System.out.println("Unknown Camera type requested: " + socket);
		}
		bufferIn = camera.getCurrentFrame();
		bufferCustom1 = new byte[bufferIn.length];
		bufferOut = new byte[bufferIn.length];
		bufferBackground = new byte[bufferIn.length];
	}

	public void cleanup() {
		camera.stop();
		camera.dispose(); // Release resources
		camera = null;
	}

	public void setX(int x) {
		bufLine = x;
	}

	public void setY(int y) {
		bufPos = y;
	}

	public void bufferCopy() {
		System.arraycopy(bufferSource, 0, bufferDestination, 0,
				bufferSource.length);
	}

	public boolean bufferCompare() {
		for (int i = 0; i < bufferSource.length; i++) {
			if (bufferDestination[i] != bufferSource[i]) {
				return false;
			}
		}
		return true;
	}

	public void setSourceBuffer(byte bufferID) {
		switch (bufferID) {
			case BUFFERID_IN :
				bufferSource = bufferIn;
				break;

			case BUFFERID_CUSTOM1 :
				bufferSource = bufferCustom1;
				break;

			case BUFFERID_OUT :
				bufferSource = bufferOut;
				break;
			case BUFFERID_BACKGR :
				bufferSource = bufferBackground;
				break;
			default :
				System.out
						.println("Unknown source buffer selected. Source unchanged "
								+ bufferSource);
				break;
		}
	}

	public void setDestinationBuffer(byte bufferID) {
		switch (bufferID) {
			case BUFFERID_IN :
				bufferDestination = bufferIn;
				break;

			case BUFFERID_CUSTOM1 :
				bufferDestination = bufferCustom1;
				break;

			case BUFFERID_OUT :
				bufferDestination = bufferOut;
				break;
			case BUFFERID_BACKGR :
				bufferDestination = bufferBackground;
				break;
			default :
				System.out
						.println("Unknown destination buffer selected. Destination unchanged "
								+ bufferDestination);
				break;
		}
	}

	public void setScratchBuffer(byte bufferID) {
		switch (bufferID) {
			case BUFFERID_IN :
				bufferScratch = bufferIn;
				break;

			case BUFFERID_CUSTOM1 :
				bufferScratch = bufferCustom1;
				break;

			case BUFFERID_OUT :
				bufferScratch = bufferOut;
				break;
			case BUFFERID_BACKGR :
				bufferScratch = bufferBackground;
				break;
			default :
				System.out
						.println("Unknown scratch buffer selected. Scratch unchanged "
								+ bufferDestination);
				break;
		}
	}
	
	public void setPos(double X, double Y) {
		mouseX = X;
		mouseY = Y;
	}
	
	public double getPosX() {return mouseX;}
	public double getPosY() {return mouseY;}

	public void setBuffer(int x, int y) {
		bufInx = y * camera.getLineSize() * 3;
		bufInx += x * 3;
	}

	public void setBufferRGB(byte[] RGB) {
		bufferDestination[bufInx + 0] = RGB[0];
		bufferDestination[bufInx + 1] = RGB[1];
		bufferDestination[bufInx + 2] = RGB[2];
	}

	public void setBufferRGB(int x, int y, byte[] RGB) {
		setBuffer(x, y);
		bufferDestination[bufInx + 0] = RGB[0];
		bufferDestination[bufInx + 1] = RGB[1];
		bufferDestination[bufInx + 2] = RGB[2];
	}

	public void getBufferRGB(byte[] RGB) {
		RGB[0] = bufferSource[bufInx + 0];
		RGB[1] = bufferSource[bufInx + 1];
		RGB[2] = bufferSource[bufInx + 2];
	}
	public byte[] getPixel() {
		byte[] temp = new byte[3];
		temp[0] = bufferSource[bufInx + 0];
		temp[1] = bufferSource[bufInx + 1];
		temp[2] = bufferSource[bufInx + 2];
		return temp;
		
	}
	public void getBufferRGB(int x, int y, byte[] RGB) {
		setBuffer(x, y);
		getBufferRGB(RGB);
	}
	
	public void resetBuffer(){
		System.arraycopy(bufferIn, 0, bufferOut, 0,
				bufferIn.length);		
	}

	// TODO This method should be blocking and should wait for a signal from the
	// usertask
	public synchronized byte[] getBuffer() {
		return bufferOut;
	}

	public Camera getCamera() {
		return camera;
	}
	
	// used
	//Use the normalized RGB color model
	public void normRGB(int threshold) {
		int r=0,g=0,b=0;
		//threshold = 0;
		int totalRGB = (bufferSource[bufInx + 0]& 0xFF)
						+ (bufferSource[bufInx + 1]& 0xFF)
						+ (bufferSource[bufInx + 2]& 0xFF);
		if(totalRGB != 0) {		
			r =  (int)(((double)(bufferSource[bufInx + 0]& 0xFF) / totalRGB)*255);
			g =  (int)(((double)(bufferSource[bufInx + 1]& 0xFF) / totalRGB)*255);
			b =  (int)(((double)(bufferSource[bufInx + 2]& 0xFF) / totalRGB)*255);
			
			if (r>threshold){
				bufferSource[bufInx + 0] = (byte) r;
			}
			else {
				bufferSource[bufInx + 0] = (byte) 0;
			}
			if (g>threshold){
				bufferSource[bufInx + 1] = (byte) g;
			}
			else {
				bufferSource[bufInx + 1] = (byte) 0;
			}
			if (b>threshold){
				bufferSource[bufInx + 2] = (byte) b;
			}
			else {
				bufferSource[bufInx + 2] = (byte) 0;
			}

//			System.out.println("r = "+bufferSource[bufInx + 0]
//			+ r+"g = "+bufferSource[bufInx + 1]+g
//			+"b = "+bufferSource[bufInx + 2]+b);			
		}	
		else {
			bufferSource[bufInx + 0] = (byte)0; 
			bufferSource[bufInx + 1] = (byte)0;
			bufferSource[bufInx + 2] = (byte)0;
		}
	}

	
	public void addToBackground(int n) {
		if (n!=0) {
			bufferBackground[bufInx + 0] = (byte) (int)((double)((int)(bufferBackground[bufInx + 0]&0xFF)*n + (int)(bufferSource[bufInx + 0]&0xFF))/(double)(n+1));
			bufferBackground[bufInx + 1] = (byte) (int)((double)((int)(bufferBackground[bufInx + 1]&0xFF)*n + (int)(bufferSource[bufInx + 1]&0xFF))/(double)(n+1));
			bufferBackground[bufInx + 2] = (byte) (int)((double)((int)(bufferBackground[bufInx + 2]&0xFF)*n + (int)(bufferSource[bufInx + 2]&0xFF))/(double)(n+1));
		} else {
			bufferBackground[bufInx + 0] = (byte) (int)(bufferSource[bufInx + 0]&0xFF);
			bufferBackground[bufInx + 1] = (byte) (int)(bufferSource[bufInx + 1]&0xFF);
			bufferBackground[bufInx + 2] = (byte) (int)(bufferSource[bufInx + 2]&0xFF);
			
		}
		
	}
	//Use the color model HSI
	public void HSI(int threshold) {
		double H=0,S=0,I=0;
		double R=0,G=0,B=0;
		
		R=(double)(bufferSource[bufInx+0]& 0xFF);
		G=(double)(bufferSource[bufInx+1]& 0xFF);
		B=(double)(bufferSource[bufInx+2]& 0xFF);
					
		if((R*2-G-B) != 0){
			H = Math.atan(Math.sqrt(3)*(G-B)/(2*R-G-B));
			if(H>threshold){
				bufferSource[bufInx+2] = (byte)H;
			} else{
				bufferSource[bufInx+2]= (byte)0;
			}
		} else{
			bufferSource[bufInx+2]= (byte)0;
		}
		
		S = 1- (Math.min(Math.min(R,G),B)/(R+G+B));
		//S = 0;
		if(S>threshold){
			bufferSource[bufInx+1] = (byte)S;
		} else{
			bufferSource[bufInx+1]= (byte)0;
		}		
		
		I = R+G+B;		
		if(I>threshold){
			bufferSource[bufInx+0] = (byte)I;
		} else{
			bufferSource[bufInx+0]= (byte)0;
		}	
	}
	//Use the color model l1 l2 l3
	public void l1l2l3(int threshold){
		int l1=0,l2=0,l3=0;
		
		int totalsquare=((bufferSource[bufInx+0]& 0xFF)-(bufferSource[bufInx+1]& 0xFF))*((bufferSource[bufInx+0]& 0xFF)-(bufferSource[bufInx+1]& 0xFF)) + 
						((bufferSource[bufInx+0]& 0xFF)-(bufferSource[bufInx+2]& 0xFF))*((bufferSource[bufInx+0]& 0xFF)-(bufferSource[bufInx+2]& 0xFF)) + 
						((bufferSource[bufInx+1]& 0xFF)-(bufferSource[bufInx+2]& 0xFF))*((bufferSource[bufInx+1]& 0xFF)-(bufferSource[bufInx+2]& 0xFF)) ;

		if (totalsquare!=0){
			l1 = (int) (((double) (((bufferSource[bufInx+0]& 0xFF)-(bufferSource[bufInx+1]& 0xFF)) * ((bufferSource[bufInx+0]& 0xFF)-(bufferSource[bufInx+1]& 0xFF))) / totalsquare)*255) ;
			l2 = (int) (((double) (((bufferSource[bufInx+0]& 0xFF)-(bufferSource[bufInx+2]& 0xFF)) * ((bufferSource[bufInx+0]& 0xFF)-(bufferSource[bufInx+2]& 0xFF))) / totalsquare)*255) ;
			l3 = (int) (((double) (((bufferSource[bufInx+1]& 0xFF)-(bufferSource[bufInx+2]& 0xFF)) * ((bufferSource[bufInx+1]& 0xFF)-(bufferSource[bufInx+2]& 0xFF))) / totalsquare)*255) ;
			if (l1>threshold){
				bufferSource[bufInx + 0] = (byte)l1;
			}
			else {
				bufferSource[bufInx + 0] = (byte) 0;
			}
			if (l2>threshold){
				bufferSource[bufInx + 1] = (byte)l2;
			}
			else {
				bufferSource[bufInx + 1] = (byte) 0;
			}
			if (l3>threshold){
				bufferSource[bufInx + 2] = (byte)l3;
			}
			else {
				bufferSource[bufInx + 2] = (byte) 0;
			}						
		}
		else {
			bufferSource[bufInx + 0] = (byte)0; 
			bufferSource[bufInx + 1] = (byte)0;
			bufferSource[bufInx + 2] = (byte)0;
		}
	}
	
	//Use the color model c1 c2 c3
	public void c1c2c3(int threshold) {
		double c1=0,c2=0,c3=0;
		double R=0,G=0,B=0;
		
		R=(double)(bufferSource[bufInx+0]& 0xFF);
		G=(double)(bufferSource[bufInx+1]& 0xFF);
		B=(double)(bufferSource[bufInx+2]& 0xFF);
					
		if(G!=0||B!=0){
			c1 = Math.atan(R/Math.max(G,B))*255;
			if(c1>=threshold){
				bufferSource[bufInx+0]=(byte)c1;
			}else{
				bufferSource[bufInx+0]=(byte)0;
			}			
		} else{
			bufferSource[bufInx+0]=0;
		}
		if(R!=0||B!=0){
			c2 = Math.atan(G/Math.max(R,B))*255;
			if(c2>=threshold){
				bufferSource[bufInx+1]=(byte)c2;
			}else{
				bufferSource[bufInx+1]=(byte)0;
			}			
		} else{
			bufferSource[bufInx+1]=0;
		}
		if(R!=0||G!=0){
			c3 = Math.atan(B/Math.max(R,G))*255;
			if(c3>=threshold){
				bufferSource[bufInx+2]=(byte)c3;
			}else{
				bufferSource[bufInx+2]=(byte)0;
			}
		} else{			
			bufferSource[bufInx+2]=0;			
		}		
	}
//		System.out.println("r = "+bufferSource[bufInx+0]+"  g = "+bufferSource[bufInx+1]+"  b = "+bufferSource[bufInx+2]);
	
	/*
	 * This methods set the convolution matrix according to it's input argument:
	 * matrixType this should be one of the statics defined above. Setting the
	 * convolution matrix means: convolutionMatrix is filled with values for the
	 * selected matrix, normalizeFactor is set for the selected matrix
	 * matrixNegative is set for the selected matrix
	 * 
	 * this method returns a byte array with the dimensions of the kernel:
	 * dimensions[0] = X dimension dimensions[1] = Y dimension
	 */
	public byte[] setConvolutionMatrix(byte matrixType) {
		byte[] dimensions = new byte[2];

		switch (matrixType) {
			
			case STAMP_5X5 :
				convolutionMatrix = new byte[][]{{1, 1, 1,1,1}, {1, 1, 1,1,1},
					{1, 1, 1,1,1},{1, 1, 1,1,1},{1, 1, 1,1,1}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 25;
				matrixNegative = true;
				break;
			case STAMP2_5X5 :
				convolutionMatrix = new byte[][]{{-1, 1, -1,1,-1}, {1, -1, 1,-1,1},
					{-1, 1, -1,1,-1},{1, -1, 1,-1,1},{-1, 1, -1,1,-1}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 25;
				matrixNegative = true;
				break;

			case LAP_3X3 : // laplacian 3x3

				convolutionMatrix = new byte[][]{{0, -1, 0}, {-1, 4, -1},
						{0, -1, 0}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 1;
				matrixNegative = true;
				break;

			case LAP_5X5 : // laplacian 5x5, this one is perhaps not correct,
				// use at own risk.

				dimensions[0] = 5;
				dimensions[1] = 5;
				convolutionMatrix = new byte[dimensions[0]][dimensions[1]];

				for (int i = 0; i < 5; i++) {
					for (int j = 0; j < 5; j++) {
						convolutionMatrix[i][j] = 1;
					}
				}
				convolutionMatrix[2][2] = 24;
				normalizeFactor = 48;
				matrixNegative = false;
				break;

			case GAUSS_3X3 : // 3x3 gaussian kernel

				convolutionMatrix = new byte[][]{{0, 1, 0}, {1, 4, 1},
						{0, 1, 0}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 8;
				matrixNegative = false;

				break;

			case FIRST_X_3X3 : // X direction Sobel mask

				convolutionMatrix = new byte[][]{{-1, -2, -1}, {0, 0, 0},
						{1, 2, 1}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 4;
				matrixNegative = true;
				break;

			case FIRST_Y_3X3 : // Y direction Sobel mask

				convolutionMatrix = new byte[][]{{-1, 0, 1}, {-2, 0, 2},
						{-1, 0, 1}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 4;
				matrixNegative = true;
				break;

			case LOG_5X5_A : // 5x5 mexican hat: laplacian 3x3 * gaussian 3x3

				convolutionMatrix = new byte[][]{{0, -1, -2, -1, 0},
						{-1, 0, 2, 0, -1}, {-2, 2, 8, 2, -2},
						{-1, 0, 2, 0, -1}, {0, -1, -2, -1, 0}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 8;
				matrixNegative = true;
				break;

			case LOG_5X5_B : // 5x5 mexican hat: laplacian 3x3 * gaussian 3x3,
							 // smaller
				// sigma then the one above

				convolutionMatrix = new byte[][]{{0, 0, -1, 0, 0},
						{0, -1, -2, -1, 0}, {-1, -2, 16, -2, -1},
						{0, -1, -2, -1, 0}, {0, 0, -1, 0, 0}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 16;
				matrixNegative = true;
				break;

			case GAUSS_5X1_X : // 5X1 First derivative of the gaussian
				// kernel in X direction, sigma = 1

				convolutionMatrix = new byte[][]{{3}, {6}, {0}, {-6}, {-3}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 10;
				matrixNegative = true;
				break;

			case GAUSS_1X5_Y : // 1X5 First derivative of the gaussian
				// kernel in the Y direction, sigma = 1

				convolutionMatrix = new byte[][]{{3, 6, 0, -6, -3}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 10;
				matrixNegative = true;
				break;

			case GAUSS_5X5_XY : // 5X5 Gaussian kernel derived in X once
				// and in Y once. sigma = 1

				convolutionMatrix = new byte[][]{{1, 3, 0, -3, -1},
						{3, 7, 0, -7, -3}, {0, 0, 0, 0, 0}, {-3, -7, 0, 7, 3},
						{-1, -3, 0, 3, 1}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 10;
				matrixNegative = true;
				break;

			case GAUSS_5X1_XX : // 5X1 Second derivative Gaussian kernel
				// in the X direction, sigma = 1

				convolutionMatrix = new byte[][]{{2}, {0}, {-5}, {0}, {2}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 5;
				matrixNegative = true;
				break;

			case GAUSS_1X5_YY : // 1X5 Second derivative Gaussian kernel
				// in the Y direction, sigma = 1

				convolutionMatrix = new byte[][]{{2, 0, -5, 0, 2}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 5;
				matrixNegative = true;
				break;

			case GAUSS_5X5 : // 5X5 gaussian kernel, sigma = 1

				convolutionMatrix = new byte[][]{{0, 1, 1, 1, 0},
						{1, 4, 6, 4, 1}, {1, 6, 10, 6, 1}, {1, 4, 6, 4, 1},
						{0, 1, 1, 1, 0}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 62;
				matrixNegative = false;
				break;

			case GAUSS_5X1 : // 5X1 X-component of the gaussian kernel above

				convolutionMatrix = new byte[][]{{1}, {6}, {10}, {6}, {1}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 24;
				matrixNegative = false;
				break;

			case GAUSS_1X5 : // 1X5 Y-component of the gaussian kernel above

				convolutionMatrix = new byte[][]{{1, 6, 10, 6, 1}};

				dimensions[0] = (byte) convolutionMatrix.length;
				dimensions[1] = (byte) convolutionMatrix[0].length;

				normalizeFactor = 24;
				matrixNegative = false;
				break;
				
			case POS_ALL : // Determine the position
				break;

		}
		return dimensions;
	}

	/*
	 * This method convolves the pixel in the image set by setX() and setY()
	 * with the kernel selected with setConvolutionMatrix(). The source of it's
	 * input image is sourceBuffer. The boolean color tells whether the
	 * convolution should be done to all the three different color values (RGB)
	 * of the pixel seperately or for the three color valued averaged, to get a
	 * black and white image. Setting this true is perhaps useful for blurring
	 * with a gaussian kernel. the boolean save determines if the computed value
	 * for the pixel is saved in destinationBuffer or not. The calculated value
	 * is always put in the array lastPixel, which contains three the same
	 * values if color==false. The value in lastPixel is not clipped, because
	 * this is an int array. The values going back into the buffer array are
	 * nevertheless clipped between 0 and 255 (byte size) When the
	 * matrixNegative is set to true, then the computed answer is shifted,
	 * effectively putting 0 in the middle (so at 128), this way we have an
	 * equal part of negative and positive results, because our clipping is done
	 * between 0 and 255 we need to do the shift, else all calculated negative
	 * results are clipped away, we don't want that.
	 * 
	 * NOTE! convolve doesn't check whether you are convolving on an edge, and
	 * that therefore pixels might not be available, checking for this should be
	 * done in the method calling convolve(), this is why setConvolutionMatrix
	 * return dimensions.
	 */
	public void convolve(boolean color, boolean save) {
		int R = 0, G = 0, B = 0;
		int P = 0, ZW = 0;

		int xSize = convolutionMatrix.length;
		int ySize = convolutionMatrix[0].length;

		// loop through the kernel
		for (int i = 0; i < xSize; i++) {
			for (int j = 0; j < ySize; j++) {
				// set the index to the pixel to calculate
				setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
						+ (int) (ySize / 2));

				// color keeps the RGB seperated
				if (color) {
					// Get rid of the byte's sign with the & operator
					R += (bufferSource[bufInx + 0] & 0xFF)
							* convolutionMatrix[i][j];
					B += (bufferSource[bufInx + 1] & 0xFF)
							* convolutionMatrix[i][j];
					G += (bufferSource[bufInx + 2] & 0xFF)
							* convolutionMatrix[i][j];
				} else {
					// Add all color channels. The value will be divided later.
					ZW = (bufferSource[bufInx + 0] & 0xFF)
							+ (bufferSource[bufInx + 1] & 0xFF)
							+ (bufferSource[bufInx + 2] & 0xFF);

					P += ZW * convolutionMatrix[i][j];
				}
			}
		}
		// Reset the current location to the convolved pixel
		setBuffer(bufLine, bufPos);

		if (color) {
			// Normalize
			lastPixel[0] = R / normalizeFactor;
			lastPixel[1] = B / normalizeFactor;
			lastPixel[2] = G / normalizeFactor;
		} else {
			// normalize (3 is from the black and white conversion)
			P = P / (normalizeFactor * 3);
			lastPixel[0] = P;
			lastPixel[1] = P;
			lastPixel[2] = P;
		}

		// save if necessary
		if (save) {
			// if matrix contains negative values, then shift and clip
			if (matrixNegative) {
				if (color) {
					bufferDestination[bufInx + 0] = (byte) Math.max(0, Math
							.min(255, lastPixel[0] + Byte.MAX_VALUE));
					bufferDestination[bufInx + 1] = (byte) Math.max(0, Math
							.min(255, lastPixel[1] + Byte.MAX_VALUE));
					bufferDestination[bufInx + 2] = (byte) Math.max(0, Math
							.min(255, lastPixel[2] + Byte.MAX_VALUE));
				} else {
					bufferDestination[bufInx + 0] = (byte) Math.max(0, Math
							.min(255, lastPixel[0] + Byte.MAX_VALUE));
					bufferDestination[bufInx + 1] = (byte) Math.max(0, Math
							.min(255, lastPixel[0] + Byte.MAX_VALUE));
					bufferDestination[bufInx + 2] = (byte) Math.max(0, Math
							.min(255, lastPixel[0] + Byte.MAX_VALUE));
				}
			} else {
				// clip values to 0 ... 255
				if (color) {
					bufferDestination[bufInx + 0] = (byte) Math.max(0, Math
							.min(255, lastPixel[0]));
					bufferDestination[bufInx + 1] = (byte) Math.max(0, Math
							.min(255, lastPixel[1]));
					bufferDestination[bufInx + 2] = (byte) Math.max(0, Math
							.min(255, lastPixel[2]));
				} else {
					bufferDestination[bufInx + 0] = (byte) Math.max(0, Math
							.min(255, lastPixel[0]));
					bufferDestination[bufInx + 1] = (byte) Math.max(0, Math
							.min(255, lastPixel[0]));
					bufferDestination[bufInx + 2] = (byte) Math.max(0, Math
							.min(255, lastPixel[0]));
				}
			}
		}
	}

	public void motion(int blocksize) {
		int avgBuf1 = 0, avgBuf2 = 0;

		int xSize = blocksize;
		int ySize = blocksize;
		int threshold = 20;

		for (int i = 0; i < xSize; i++) {
			for (int j = 0; j < ySize; j++) {
				setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
						+ (int) (ySize / 2));

				// Zwart Wit
				avgBuf2 += ((bufferSource[bufInx + 0] & 0xFF)
						+ (bufferSource[bufInx + 1] & 0xFF) + (bufferSource[bufInx + 2] & 0xFF));
				avgBuf1 += ((bufferScratch[bufInx + 0] & 0xFF)
						+ (bufferScratch[bufInx + 1] & 0xFF) + (bufferScratch[bufInx + 2] & 0xFF));
			}
		}
		setBuffer(bufLine, bufPos);

		avgBuf2 = avgBuf2 / (xSize * ySize * 3);
		avgBuf1 = avgBuf1 / (xSize * ySize * 3);

		// threshold
		if (Math.abs(avgBuf2 - avgBuf1) >= threshold
				&& Math.abs(avgBuf2 - avgBuf1) <= threshold + 10) {
			bufferDestination[bufInx + 0] = (byte) 255;
			bufferDestination[bufInx + 1] = (byte) 0;
			bufferDestination[bufInx + 2] = (byte) 0;
		} else if (Math.abs(avgBuf2 - avgBuf1) > threshold + 10) {
			bufferDestination[bufInx + 0] = (byte) 0;
			bufferDestination[bufInx + 1] = (byte) 255;
			bufferDestination[bufInx + 2] = (byte) 0;
		} else {
			bufferDestination[bufInx + 0] = bufferSource[bufInx + 0];
			bufferDestination[bufInx + 1] = bufferSource[bufInx + 1];
			bufferDestination[bufInx + 2] = bufferSource[bufInx + 2];
		}
	}
public boolean subtract2(int blocksize) {
		int avgBuf1 = 0, avgBuf2 = 0;

		int xSize = blocksize;
		int ySize = blocksize;
		int threshold = 10;
		boolean returnValue = false;
		
		for (int i = 0; i < xSize; i++) {
			for (int j = 0; j < ySize; j++) {
				setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
						+ (int) (ySize / 2));
				
				int v1 = Math.abs(bufferSource[bufInx + 0] - bufferScratch[bufInx + 0]);
				int v2 = Math.abs(bufferSource[bufInx + 1] - bufferScratch[bufInx + 1]);
				int v3 = Math.abs(bufferSource[bufInx + 2] - bufferScratch[bufInx + 2]);
			
				
				if (v1>threshold&&v2>threshold&&v3>threshold) {
					v1=255;//bufferSource[bufInx + 0]+50;
					v2=0;//bufferSource[bufInx + 1]+50;
					v3=0;//bufferSource[bufInx + 2]+50;
					returnValue = true;
				
				} else if (v1<-threshold&&v2<-threshold&&v3<-threshold) {
					v1=0;//bufferSource[bufInx + 0]+50;
					v2=0;//bufferSource[bufInx + 1]+50;
					v3=255;//bufferSource[bufInx + 2]+50;
					returnValue = true;
				
				} else {
					v1=0;//(int)Math.floor((int)bufferSource[bufInx + 0]/3);
					v2=0;//(int)Math.floor((int)bufferSource[bufInx + 1]/3);
					v3=0;//(int)Math.floor((int)bufferSource[bufInx + 2]/3);	
					returnValue = false;
					
				}
				if (v1<0) { v1=0; }
				if (v2<0) { v2=0; }
				if (v3<0) { v3=0; }
				
				if (v1>255) { v1=255; }
				if (v2>255) { v2=255; }
				if (v3>255) { v3=255; }
				
				bufferDestination[bufInx + 0] = (byte)v1;
				bufferDestination[bufInx + 1] = (byte)v2;
				bufferDestination[bufInx + 2] = (byte)v3;

			}
		}
		return returnValue;
	}
public boolean sense(int val,int blocksize) {
	
	if (val==1) {
		
		return subtract2(blocksize);
	}	else {
		
		return subtract2(blocksize);	
	}
	
	
	
}
public void subtract(int blocksize) {
		int avgBuf1 = 0, avgBuf2 = 0;

		int xSize = blocksize;
		int ySize = blocksize;
		int threshold = 10;
		
		for (int i = 0; i < xSize; i++) {
			for (int j = 0; j < ySize; j++) {
				setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
						+ (int) (ySize / 2));
				
				int v1 = Math.abs(bufferSource[bufInx + 0] - bufferScratch[bufInx + 0]);
				int v2 = Math.abs(bufferSource[bufInx + 1] - bufferScratch[bufInx + 1]);
				int v3 = Math.abs(bufferSource[bufInx + 2] - bufferScratch[bufInx + 2]);
				
				
			
			
				
			
				
				if (v1>threshold&&v2>threshold&&v3>threshold) {
					v1=255;//bufferSource[bufInx + 0]+50;
					v2=0;//bufferSource[bufInx + 1]+50;
					v3=0;//bufferSource[bufInx + 2]+50;
				
				} else if (v1<-threshold&&v2<-threshold&&v3<-threshold) {
					v1=0;//bufferSource[bufInx + 0]+50;
					v2=0;//bufferSource[bufInx + 1]+50;
					v3=255;//bufferSource[bufInx + 2]+50;
				
				} else {
					v1=0;//(int)Math.floor((int)bufferSource[bufInx + 0]/3);
					v2=0;//(int)Math.floor((int)bufferSource[bufInx + 1]/3);
					v3=0;//(int)Math.floor((int)bufferSource[bufInx + 2]/3);	
				
					
				}
				if (v1<0) { v1=0; }
				if (v2<0) { v2=0; }
				if (v3<0) { v3=0; }
				
				if (v1>255) { v1=255; }
				if (v2>255) { v2=255; }
				if (v3>255) { v3=255; }
				
				bufferDestination[bufInx + 0] = (byte)v1;
				bufferDestination[bufInx + 1] = (byte)v2;
				bufferDestination[bufInx + 2] = (byte)v3;

			}
		}
		
	}

public void subtract3(int blocksize) {
	int avgBuf1 = 0, avgBuf2 = 0;

	int xSize = blocksize;
	int ySize = blocksize;
	int threshold = 40;
	int n = 2;
	for (int i = 0; i < xSize; i++) {
		for (int j = 0; j < ySize; j++) {
			setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
					+ (int) (ySize / 2));
			
			int v1 = Math.abs(bufferSource[bufInx + 0]& 0xFF - bufferScratch[bufInx + 0]& 0xFF);
			int v2 = Math.abs(bufferSource[bufInx + 1]& 0xFF - bufferScratch[bufInx + 1]& 0xFF);
			int v3 = Math.abs(bufferSource[bufInx + 2]& 0xFF - bufferScratch[bufInx + 2]& 0xFF);
			//bufferScratch[bufInx + 0] = (byte)((int)(double)(n*bufferScratch[bufInx + 0] + bufferSource[bufInx + 0])/(double)(n+1));
			//bufferScratch[bufInx + 1] = (byte)((int)(double)(n*bufferScratch[bufInx + 1] + bufferSource[bufInx + 1])/(double)(n+1));
			//bufferScratch[bufInx + 2] = (byte)((int)(double)(n*bufferScratch[bufInx + 2] + bufferSource[bufInx + 2])/(double)(n+1));
			/*if (Math.abs(v1)>0||Math.abs(v2)>0||Math.abs(v3)>0) {
				//System.out.println("SCRATCH: "+Math.abs(bufferScratch[bufInx + 0]& 0xFF) + " - " + Math.abs(bufferScratch[bufInx + 1]& 0xFF) + " - " + Math.abs(bufferScratch[bufInx + 2]& 0xFF));
				//System.out.println("SOURCE:  " + Math.abs(bufferSource[bufInx + 0]& 0xFF) + " - " + Math.abs(bufferSource[bufInx + 1]& 0xFF) + " - " + Math.abs(bufferSource[bufInx + 2]& 0xFF));
				//System.out.println();
				bufferSource[bufInx + 0] = (byte)255;
				bufferSource[bufInx + 1] = (byte)255;
				bufferSource[bufInx + 2] = (byte)255;
			} else {
				bufferSource[bufInx + 0] = (byte)0;
				bufferSource[bufInx + 1] = (byte)0;
				bufferSource[bufInx + 2] = (byte)0;
				
				
			}*/
			if (Math.abs(v1)>threshold&&Math.abs(v2)>threshold&&Math.abs(v3)>threshold) {
				bufferSource[bufInx + 0] = (byte)255;
				bufferSource[bufInx + 1] = (byte)255;
				bufferSource[bufInx + 2] = (byte)255;
			} else {
				bufferSource[bufInx + 0] = (byte)0;
				bufferSource[bufInx + 1] = (byte)0;
				bufferSource[bufInx + 2] = (byte)0;
				
			}
			
			
		}
	}
	
}
public int[] getRGB(int x, int y) {
	setBuffer(x,y);
	int[] return_ar = new int[3];
	return_ar[0] = bufferSource[bufInx + 0]&0xFF;
	return_ar[1] = bufferSource[bufInx + 1]&0xFF;
	return_ar[2] = bufferSource[bufInx + 2]&0xFF;
	return return_ar;
}
public void setRGB(int x, int y, int[] RGB) {
	setBuffer(x,y);
	
	bufferSource[bufInx + 0] = (byte)RGB[0];
	bufferSource[bufInx + 1] = (byte)RGB[1];
	bufferSource[bufInx + 2] = (byte)RGB[2];
	
}
public void subtract4(int blocksize, int threshold) {
	 //20 for RGB // 2 for nRGB
	int xSize = blocksize;
	int ySize = blocksize;
	int n = 0;
	threshold = 20;
	//int sourceR,sourceG,sourceB, bufferR,bufferG,bufferB;
	for (int i = 0; i < xSize; i++) {
		for (int j = 0; j < ySize; j++) {
			setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j + (int) (ySize / 2));
			int v1 = Math.abs((bufferSource[bufInx + 0]&0xFF) - (bufferBackground[bufInx + 0]&0xFF));
			int v2 = Math.abs((bufferSource[bufInx + 1]&0xFF) - (bufferBackground[bufInx + 1]&0xFF));
			int v3 = Math.abs((bufferSource[bufInx + 2]&0xFF) - (bufferBackground[bufInx + 2]&0xFF));
			
			//System.out.println(v1 + " - " + v2 + " - " + v3 );
			
			if (v1>threshold&&v2>threshold&&v3>threshold) {
				v1=255;
				v2=255;
				v3=255;				
			} else {
				//v1=bufferSource[bufInx + 0];
				//v2=bufferSource[bufInx + 1];
				//v3=bufferSource[bufInx + 2];
				v1=0;
				v2=0;
				v3=0;
			}
			//System.out.println("B " + bufferBackground[bufInx+0] + " - " +bufferBackground[bufInx+1] + " - " +bufferBackground[bufInx+2] );
			if (n>0) {
				bufferBackground[bufInx + 0 ] = (byte) ((int)((double)((n * (bufferBackground[bufInx + 0 ]& 0xFF)) + (bufferSource[bufInx + 0 ]&0xFF)) / (double)(n+1)));  
				bufferBackground[bufInx + 1 ] = (byte) ((int)((double)((n * (bufferBackground[bufInx + 1 ]& 0xFF)) + (bufferSource[bufInx + 1 ]&0xFF)) / (double)(n+1)));
				bufferBackground[bufInx + 2 ] = (byte) ((int)((double)((n * (bufferBackground[bufInx + 2 ]& 0xFF)) + (bufferSource[bufInx + 2 ]&0xFF)) / (double)(n+1)));
			}
			//System.out.println("A " + bufferBackground[bufInx+0] + " - " +bufferBackground[bufInx+1] + " - " +bufferBackground[bufInx+2] );
			bufferDestination[bufInx + 0] = (byte)v1;//(byte)v1;
			bufferDestination[bufInx + 1] = (byte)v2;//(byte)v2;
			bufferDestination[bufInx + 2] = (byte)v3;//(byte)v3;
		}
	}
	
	
	
	
}
// ******* added
public void fillPixel(int blocksize, int color) {
		int avgBuf1 = 0, avgBuf2 = 0;

		int xSize = blocksize;
		int ySize = blocksize;
		int threshold = 10;
		
		for (int i = 0; i < xSize; i++) {
			for (int j = 0; j < ySize; j++) {
				setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
						+ (int) (ySize / 2));
				
				bufferDestination[bufInx + 0] = (byte)color;
				bufferDestination[bufInx + 1] = (byte)color;
				bufferDestination[bufInx + 2] = (byte)color;

			}
		}
		
	}


	/*
	 * This method determines for the pixel set with setX() and setY() in the
	 * bufferSource whether this is pixel is on a zero crossing. Because of the
	 * shifting done in convolve() the zero crossing is actually between 127 and
	 * 128. background determines if we want the original image visible where
	 * there is no zero crossing or just a black image. threshold gives the
	 * value to determine when something is a zero crossing, the absolute
	 * difference between the two pixel where there is a zero crossing is
	 * thresholded. Up until now we found that a low threshold is enough (up to
	 * 5) but this depends on how much the scale of an image is changed by your
	 * convolution.
	 */
	public void markZeroCrossing(boolean background, int threshold) {
		boolean zeroCrossing = false;

		int sp;
		int tp;

		// original pixel
		setBuffer(bufLine, bufPos);
		sp = bufferSource[bufInx] & 0xFF;

		// loop through all neighbouring pixels
		for (int i = -1; i < 2; i++) {
			for (int j = -1; j < 2; j++) {
				setBuffer(bufLine + i, bufPos + j);
				tp = bufferSource[bufInx] & 0xFF;

				// we don't need to check with ourselves
				if (i == 0 && j == 0) {
					continue;
				}
				// if the absolute difference between the pixel is large enough
				// (indicating
				// a peak in the first derivative) and one of the pixels is
				// above zero and the
				// other below, and of those two, we are the closest to zero,
				// then we have
				// a zero crossing
				if (Math.abs(sp - tp) > threshold
						&& ((sp >= 128 && tp < 128 && sp - 128 <= Math
								.abs(tp - 127)) || (sp < 128 && tp >= 128 && tp - 128 >= Math
								.abs(sp - 127)))) {
					zeroCrossing = true;
				}
			}
		}

		setBuffer(bufLine, bufPos);
		setSourceBuffer(BUFFERID_IN);

		if (zeroCrossing) {
			// Edge pixels are colored
			bufferDestination[bufInx + 0] = (byte) 255;
			bufferDestination[bufInx + 1] = (byte) 0;
			bufferDestination[bufInx + 2] = (byte) 0;
		} else {
			if (background) {
				// Non-edge pixels are original video
				bufferDestination[bufInx + 0] = bufferIn[bufInx + 0];
				bufferDestination[bufInx + 1] = bufferIn[bufInx + 1];
				bufferDestination[bufInx + 2] = bufferIn[bufInx + 2];
			} else {
				// Non-edge pixels are black
				bufferDestination[bufInx + 0] = (byte) 0;
				bufferDestination[bufInx + 1] = (byte) 0;
				bufferDestination[bufInx + 2] = (byte) 0;
			}
		}
	}

	/*
	 * This method determines the first derivative of a pixel in the
	 * bufferSource, given by setX() and setY(). It convolves with the Sobel
	 * masks, FIRST_X and FIRST_Y for this and then computes the magnitude. save
	 * indicates whether the output should be saved to bufferDestination
	 * threshold gives a threshold for what is considered an edge and what not.
	 * Setting this to 0 indicates no threshold, and then magnitude is clipped
	 * and saved. The method returns the unclipped and un squarerooted value of
	 * the magnitude (this was used in the zero crossings method)
	 */
	public int firstDerivative(boolean save, byte threshold) {
		int P1 = 0, P2 = 0, P = 0;

		// X derivative
		setConvolutionMatrix(FIRST_X_3X3);
		convolve(false, false);
		P1 = lastPixel[0];

		// Y derivative
		setConvolutionMatrix(FIRST_Y_3X3);
		convolve(false, false);
		P2 = lastPixel[0];

		// if you want to show it
		if (save) {
			setBuffer(bufLine, bufPos);

			P = (int) Math.sqrt((P1 * P1) + (P2 * P2));
			P = Math.max(0, Math.min(255, P));

			// no threshold
			if (threshold == 0) {
				bufferDestination[bufInx + 0] = (byte) P;
				bufferDestination[bufInx + 1] = (byte) P;
				bufferDestination[bufInx + 2] = (byte) P;
			} else {
				if (P > threshold) {
					bufferDestination[bufInx + 0] = (byte) 255;
					bufferDestination[bufInx + 1] = (byte) 0;
					bufferDestination[bufInx + 2] = (byte) 0;
				} else {
					bufferDestination[bufInx + 0] = bufferSource[bufInx + 0];
					bufferDestination[bufInx + 1] = bufferSource[bufInx + 1];
					bufferDestination[bufInx + 2] = bufferSource[bufInx + 2];
				}

			}
		}

		return (P1 * P1) + (P2 * P2);
	}

	/*
	 * This method computes the so called fww, which is the second derivative of
	 * a pixel in the direction of the gradient. It does this for the pixel
	 * setX() and setY() in the bufferSource. This is done by convolving with a
	 * number of Gaussian derivative kernels: GAUSS_5X1_X, GAUSS_1X5_Y,
	 * GAUSS_5X5_XY, GAUSS_5X1_XX, GAUSS_1X5_YY save indicates whether or not
	 * the output should be saved in bufferDestination the computed pixel is
	 * returned as well.
	 */
	public int gaussWW(boolean save) {
		int fx = 0, fy = 0, fxy = 0, fxx = 0, fyy = 0, P = 0;

		setConvolutionMatrix(GAUSS_5X1_X);
		convolve(false, false);
		fx = lastPixel[0];

		setConvolutionMatrix(GAUSS_1X5_Y);
		convolve(false, false);
		fy = lastPixel[0];

		setConvolutionMatrix(GAUSS_5X5_XY);
		convolve(false, false);
		fxy = lastPixel[0];

		setConvolutionMatrix(GAUSS_1X5_YY);
		convolve(false, false);
		fyy = lastPixel[0];

		setConvolutionMatrix(GAUSS_5X1_XX);
		convolve(false, false);
		fxx = lastPixel[0];

		P = (int) ((fx * fx * fxx) + (2 * fx * fy * fxy) + (fy * fy * fyy));

		// voor als je de eerste afgeleide wilt construeren
		if (save) {
			setBuffer(bufLine, bufPos);

			P = Math.max(0, Math.min(255, P + Byte.MAX_VALUE));

			bufferDestination[bufInx + 0] = (byte) P;
			bufferDestination[bufInx + 1] = (byte) P;
			bufferDestination[bufInx + 2] = (byte) P;
		}

		return P;
	}
	
	
	
	public void doSomething() {
		int threshold = 254;
		
		convolve(true, false);
		
		if (lastPixel[0]>threshold&&lastPixel[1]>threshold&&lastPixel[2]>threshold) {
			//System.out.println(lastPixel[0] + " - " + lastPixel[1] + " - " + lastPixel[2]);
			//System.out.println(normalizeFactor);
			bufferDestination[bufInx+0] = (byte)255;
			bufferDestination[bufInx+1] = (byte)0;
			bufferDestination[bufInx+2] = (byte)0;
		} else {
			bufferDestination[bufInx+0] = (byte)0;
			bufferDestination[bufInx+1] = (byte)0;
			bufferDestination[bufInx+2] = (byte)0;
			
		}
		//System.out.println(lastPixel[0]);
		
		
		
		
		/*
		for (int i = 0; i < xSize; i++) {
			for (int j = 0; j < ySize; j++) {
				setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
						+ (int) (ySize / 2));

							// kleur (Get rid of the byte's sign with the & operator)
				if ((bufferSource[bufInx + 0] & 0xFF) * convX[i][j] > 0){
					sum++;
					
				}
			

			}
		}
		if (sum>0) {
			
					(bufferSource[bufInx + 0] & 0xFF) * convX[i][j] > 0)
				
	
				}
			}
		}*/
	}


	public void optFlow() {
		int[][] convX = {{1, 2, 1}, 
						 {0, 0, 0}, 
						 {-1, -2, -1}};
		int[][] convY = {{1, 0, -1}, 
						 {2, 0, -2}, 
						 {1, 0, -1}};
		int[][] convT = {{1, 1, 1}, 
						 {1, 1, 1}, 
						 {1, 1, 1}};
		int threshold = 10;
		int xSize = convX.length;
		int ySize = convX[0].length;
		double gradientXG = 0, gradientXB = 0, gradientXR = 0, gradientYG = 0, gradientYB = 0, gradientYR = 0, gradientTG = 0, gradientTB = 0, gradientTR = 0, u = 0, v = 0, N = 0, N2 = 0;

		for (int i = 0; i < xSize; i++) {
			for (int j = 0; j < ySize; j++) {
				setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
						+ (int) (ySize / 2));

				// voor normalisatie naar kleine rgb
				N = (bufferSource[bufInx + 0] & 0xFF)
						+ (bufferSource[bufInx + 1] & 0xFF)
						+ (bufferSource[bufInx + 2] & 0xFF);
				N2 = (bufferScratch[bufInx + 0] & 0xFF)
						+ (bufferScratch[bufInx + 1] & 0xFF)
						+ (bufferScratch[bufInx + 2] & 0xFF);

				// kleur (Get rid of the byte's sign with the & operator)
				gradientXG += ((bufferSource[bufInx + 0] & 0xFF) / N + (bufferScratch[bufInx + 0] & 0xFF)
						/ N2)
						* convX[i][j];
				gradientXR += ((bufferSource[bufInx + 2] & 0xFF) / N + (bufferScratch[bufInx + 2] & 0xFF)
						/ N2)
						* convX[i][j];

				gradientYG += ((bufferSource[bufInx + 0] & 0xFF) / N + (bufferScratch[bufInx + 0] & 0xFF)
						/ N2)
						* convY[i][j];
				gradientYR += ((bufferSource[bufInx + 2] & 0xFF) / N + (bufferScratch[bufInx + 2] & 0xFF)
						/ N2)
						* convY[i][j];

				gradientTG += ((bufferSource[bufInx + 0] & 0xFF) / N - (bufferScratch[bufInx + 0] & 0xFF)
						/ N2)
						* convT[i][j];
				gradientTR += ((bufferSource[bufInx + 2] & 0xFF) / N - (bufferScratch[bufInx + 2] & 0xFF)
						/ N2)
						* convT[i][j];

			}
		}
		setBuffer(bufLine, bufPos);

		gradientXG /= 2;
		gradientXR /= 2;

		gradientYG /= 2;
		gradientYR /= 2;

		gradientTG = Math.abs(gradientTG) / 9;
		gradientTR = Math.abs(gradientTR) / 9;

		u = (gradientTG * gradientYR - gradientTR * gradientYG)
				/ (gradientXR * gradientYG - gradientXG * gradientYR);
		v = (gradientTG * gradientXR - gradientTR * gradientXG)
				/ (gradientXG * gradientYR - gradientXR * gradientYG);
		//System.out.println("u = "+(int)u);
		//System.out.println("v = "+(int)v);

		if (Math.abs(u) > Math.abs(v)) {
			//optical flow in positieve x richting blauw (rechts)
			if ((u > threshold)) {
				bufferDestination[bufInx + 0] = (byte) 0;
				bufferDestination[bufInx + 1] = (byte) 0;
				bufferDestination[bufInx + 2] = (byte) 255;
			} else {
				//optical flow in negatieve x richting paars (links)
				if ((u < -threshold)) {
					bufferDestination[bufInx + 0] = (byte) 255;
					bufferDestination[bufInx + 1] = (byte) 255;
					bufferDestination[bufInx + 2] = (byte) 255;
				} else {
					bufferDestination[bufInx + 0] = (byte) 0;
					bufferDestination[bufInx + 1] = (byte) 0;
					bufferDestination[bufInx + 2] = (byte) 0;
				}
			}
		} else {
			//optical flow in positieve y richting rood (boven)
			if ((v > threshold)) {
				bufferDestination[bufInx + 0] = (byte) 0;//255;
				bufferDestination[bufInx + 1] = (byte) 0;
				bufferDestination[bufInx + 2] = (byte) 0;
			} else {
				//optical flow in negatieve y richting groen (beneden)
				if ((v < -threshold)) {
					bufferDestination[bufInx + 0] = (byte) 0;
					bufferDestination[bufInx + 1] = (byte) 0;//255;
					bufferDestination[bufInx + 2] = (byte) 0;
				} else {
					//anders zwaart
					bufferDestination[bufInx + 0] = (byte) 0;
					bufferDestination[bufInx + 1] = (byte) 0;
					bufferDestination[bufInx + 2] = (byte) 0;
				}
			}
		}
	}

	public byte optFlowArrow() {
		int[][] convX = {{1, 2, 1}, {0, 0, 0}, {-1, -2, -1}};
		int[][] convY = {{1, 0, -1}, {2, 0, -2}, {1, 0, -1}};
		int[][] convT = {{1, 1, 1}, {1, 1, 1}, {1, 1, 1}};
		int xSize = convX.length;
		int ySize = convX[0].length;
		double gradientXG = 0, gradientXB = 0, gradientXR = 0, gradientYG = 0, gradientYB = 0, gradientYR = 0, gradientTG = 0, gradientTB = 0, gradientTR = 0, u = 0, v = 0, N = 0, N2 = 0;

		for (int i = 0; i < xSize; i++) {
			for (int j = 0; j < ySize; j++) {
				setBuffer(bufLine - i + (int) (xSize / 2), bufPos - j
						+ (int) (ySize / 2));

				// voor normalisatie naar kleine rgb
				N = (bufferSource[bufInx + 0] & 0xFF)
						+ (bufferSource[bufInx + 1] & 0xFF)
						+ (bufferSource[bufInx + 2] & 0xFF);
				N2 = (bufferScratch[bufInx + 0] & 0xFF)
						+ (bufferScratch[bufInx + 1] & 0xFF)
						+ (bufferScratch[bufInx + 2] & 0xFF);

				// kleur (Get rid of the byte's sign with the & operator)
				gradientXG += ((bufferSource[bufInx + 0] & 0xFF) / N + (bufferScratch[bufInx + 0] & 0xFF)
						/ N2)
						* convX[i][j];
				gradientXR += ((bufferSource[bufInx + 2] & 0xFF) / N + (bufferScratch[bufInx + 2] & 0xFF)
						/ N2)
						* convX[i][j];

				gradientYG += ((bufferSource[bufInx + 0] & 0xFF) / N + (bufferScratch[bufInx + 0] & 0xFF)
						/ N2)
						* convY[i][j];
				gradientYR += ((bufferSource[bufInx + 2] & 0xFF) / N + (bufferScratch[bufInx + 2] & 0xFF)
						/ N2)
						* convY[i][j];

				gradientTG += ((bufferSource[bufInx + 0] & 0xFF) / N - (bufferScratch[bufInx + 0] & 0xFF)
						/ N2)
						* convT[i][j];
				gradientTR += ((bufferSource[bufInx + 2] & 0xFF) / N - (bufferScratch[bufInx + 2] & 0xFF)
						/ N2)
						* convT[i][j];

			}
		}
		setBuffer(bufLine, bufPos);

		gradientXG /= 2;
		gradientXR /= 2;

		gradientYG /= 2;
		gradientYR /= 2;

		gradientTG = Math.abs(gradientTG) / 9;
		gradientTR = Math.abs(gradientTR) / 9;

		u = (gradientTG * gradientYR - gradientTR * gradientYG)
				/ (gradientXR * gradientYG - gradientXG * gradientYR);
		v = (gradientTG * gradientXR - gradientTR * gradientXG)
				/ (gradientXG * gradientYR - gradientXR * gradientYG);
		//System.out.println("u = "+(int)u);
		//System.out.println("v = "+(int)v);

		if (Math.abs(u) > Math.abs(v)) {
			//optical flow in positieve x richting blauw (rechts)
			if ((u > 5)) {
				return 1;
			} else {
				//optical flow in negatieve x richting paars (links)
				if ((u < -5)) {
					return 2;
				} else {
					return 0;
				}
			}
		} else {
			//optical flow in positieve y richting rood (boven)
			if ((v > 5)) {
				return 3;
			} else {
				//optical flow in negatieve y richting groen (beneden)
				if ((v < -5)) {
					return 4;
				} else {
					//anders zwaart
					return 0;
				}
			}
		}
	}

	public void drawOptFlow() {
		int N = -1, S = -1, E = -1, W = -1, p = 0, max = 0;

		for (int i = 0; i < 10; i++) {
			for (int j = 0; j < 10; j++) {
				setBuffer(bufLine - i + (int) (10 / 2), bufPos - j
						+ (int) (10 / 2));

				p = optFlowArrow();

				if (p == 1) {
					E++;
				}
				if (p == 2) {
					W++;
				}
				if (p == 3) {
					N++;
				}
				if (p == 4) {
					S++;
				}
			}
		}
		setBuffer(bufLine, bufPos);
		max = Math.max(0, Math.max(E, Math.max(W, Math.max(N, S))));
		if (E == max) {
			drawArrow((byte) bufLine, (byte) bufPos, (byte) (bufLine + 4),
					(byte) bufPos);
		} else {
			if (W == max) {
				drawArrow((byte) bufLine, (byte) bufPos, (byte) (bufLine - 4),
						(byte) bufPos);
			} else {
				if (N == max) {
					drawArrow((byte) bufLine, (byte) bufPos, (byte) bufLine,
							(byte) (bufPos + 4));
				} else {
					if (S == max) {
						drawArrow((byte) bufLine, (byte) bufPos,
								(byte) bufLine, (byte) (bufPos - 4));
					} else {
						//
					}
				}
			}
		}
	}

	// This function may be started by optFlow() and draws an Arrow between
	// coordinate 1 and 2
	// Purely horizontal / vertical arrows are visualised poorly.
	// Development was stopped because color visualisation seemed more nice at
	// some point.
	public void drawArrow(byte x1b, byte y1b, byte x2b, byte y2b) {
		// this is necessary because bytes are signed in Java
		int x1 = x1b & 0xFF;
		int x2 = x2b & 0xFF;
		int y1 = y1b & 0xFF;
		int y2 = y2b & 0xFF;

		int xtemp = x1; // the x-coordinate (in pixels) that is going to be
						// changed
		int ytemp = y1; // the y-coordinate (in pixels) that is going to be
						// changed
		int stepx = 0; // the size of the step in the x-direction
		int stepy = 0; // the size of the step in the y-direction
		int tempstepx; // how much of the step is already done
		int tempstepy;
		int a = x2 - x1; // x difference between two pixels
		int b = y2 - y1; // y difference between two pixels

		int i; // counter;
		int imax = 2; // maxsize of the arrowhead

		if (x1 > camera.getLineSize()-1 || x2 > camera.getLineSize()-1 || y1 > camera.getFrameSize()-1 || y2 > camera.getFrameSize()-1 || x1 < 0 || x2 < 0
				|| y1 < 0 || y2 < 0) {
			System.out.println("drawArrow : Out of bounds");
			System.out.println("x1 = " + x1 + "y1 = " + y1 + "x2 = " + x2
					+ "y2 = " + y2);
			return;
		}

		// pixel 0,0 is bottom-left

		// direction is up right
		if (a > 0 && b > 0 && a >= b) {
			stepx = a / b;
			stepy = 1;
		}

		// direction is up right
		if (a > 0 && b >= 0 && a < b) {
			stepx = 1;
			stepy = b / a;
		}

		// direction is up left
		if (a < 0 && b > 0 && Math.abs(a) >= b) {
			stepx = a / b; // negative
			stepy = 1;
		}

		// direction is up left
		if (a < 0 && b > 0 && Math.abs(a) < b) {
			stepx = -1;
			stepy = b / Math.abs(a);
		}

		// direction is bottom left
		if (a < 0 && b < 0 & Math.abs(a) >= Math.abs(b)) {
			stepx = -(a / b); // negative
			stepy = -1;
		}

		if (a < 0 && b < 0 && Math.abs(a) < Math.abs(b)) {
			stepx = -1;
			stepy = -(b / a);
		}

		// direction is bottom right
		if (a > 0 && b < 0 && a >= Math.abs(b)) {
			stepx = a / Math.abs(b);
			stepy = -1;
		}

		// irection is bottom right
		if (a > 0 && b < 0 && a < Math.abs(b)) {
			stepx = 1;
			stepy = b / a;
		}

		//direction is up or down
		if (a == 0 && b != 0) {
			stepx = 0;
			stepy = b;
		}

		// direction is left or right
		if (a != 0 && b == 0) {

			stepx = a;
			stepy = 0;
		}

		if (a == 0 && b == 0) {
			stepx = 0;
			stepy = 0;
		}

		tempstepx = stepx;
		tempstepy = stepy;

		//debug
		//System.out.println("x1 = " + x1 + "y1 = " + y1 + "x2 = " + x2 + "y2 =
		// " + y2);
		//System.out.println("a = " + a + "b = " + b);
		//System.out.println("stepx = " + stepx + "stepy = " + stepy +
		// "tempstepx = " + tempstepx + "tempstepy = " + tempstepy);

		// line :
		int loop = 0;
		while ((  Math.abs(xtemp-x2) > 10)    && loop < 1000    ) {
			loop++;
			setBuffer(xtemp, ytemp);
			bufferDestination[bufInx + 0] = (byte) 255;
			bufferDestination[bufInx + 1] = (byte) 255;
			bufferDestination[bufInx + 2] = (byte) 255;

			if (tempstepx > 0 && xtemp != x2) {
				xtemp++;
				tempstepx--;
			}

			if (tempstepx < 0 && xtemp != x2) {
				xtemp--;
				tempstepx++;
			}

			if (xtemp == x2) {
				tempstepx = 0;
			}

			if (tempstepx == 0 && tempstepy > 0 && ytemp != y2) {
				ytemp++;
				tempstepy--;
			}

			if (tempstepx == 0 && tempstepy < 0 && ytemp != y2) {
				ytemp--;
				tempstepy++;
			}

			if (tempstepx == 0 && tempstepy == 0) {
				tempstepx = stepx;
				tempstepy = stepy;
			}
		}

		//arrowhead :
		/*
		for (i = 1; i <= imax; i++) {

			if (a != 0 && xtemp - i * (a / Math.abs(a)) < 176
					&& xtemp - i * (a / Math.abs(a)) > 0) {
				setBuffer(xtemp - i * a / Math.abs(a), ytemp);
				bufferDestination[bufInx + 0] = (byte) 255;
				bufferDestination[bufInx + 1] = (byte) 255;
				bufferDestination[bufInx + 2] = (byte) 255;
			}
			if (b != 0 && ytemp - i * (b / Math.abs(b)) < 144
					&& ytemp - i * (b / Math.abs(b)) > 0) {
				setBuffer(xtemp, ytemp - i * (b / Math.abs(b)));
				bufferDestination[bufInx + 0] = (byte) 255;
				bufferDestination[bufInx + 1] = (byte) 255;
				bufferDestination[bufInx + 2] = (byte) 255;
			}
			if (a == 0 && xtemp - i > 0) {
				setBuffer(xtemp - i, ytemp);
				bufferDestination[bufInx + 0] = (byte) 255;
				bufferDestination[bufInx + 1] = (byte) 255;
				bufferDestination[bufInx + 2] = (byte) 255;
			}

			if (a == 0 && xtemp + i < 176) {
				setBuffer(xtemp + i, ytemp);
				bufferDestination[bufInx + 0] = (byte) 255;
				bufferDestination[bufInx + 1] = (byte) 255;
				bufferDestination[bufInx + 2] = (byte) 255;
			}
			if (b == 0 && ytemp - i > 0) {
				setBuffer(xtemp, ytemp - i);
				bufferDestination[bufInx + 0] = (byte) 255;
				bufferDestination[bufInx + 1] = (byte) 255;
				bufferDestination[bufInx + 2] = (byte) 255;
			}
			if (b == 0 && ytemp + i < 144) {
				setBuffer(xtemp, ytemp + i);
				bufferDestination[bufInx + 0] = (byte) 255;
				bufferDestination[bufInx + 1] = (byte) 255;
				bufferDestination[bufInx + 2] = (byte) 255;
			}
			
		}*/
	}
	
	public void close(int element) {
		int iMax = getCamera().getFrameSize();
		int jMax = getCamera().getLineSize();
		int oldBufLine = bufLine;
		int oldBufPos = bufPos;
		int newBufLine = bufLine-(element/2);
		int newBufPos = bufPos-(element/2);
		boolean remain = true;
		
		for (int i = 0; i < element; i++) {
			for (int j = 0; j < element; j++) {
				bufLine = newBufLine+i;
				bufPos = newBufPos+j;
				setBuffer(bufLine,bufPos);
				if(bufPos>=0&&bufPos<bufferDestination.length/(jMax*3)&&bufLine>=0&&bufLine<bufferDestination.length/(iMax*3)) {
					int v1 = bufferDestination[bufInx + 0]&0xFF;
					if(v1!=255) {
						remain = false;
						break;
					}
				}
			}
			if(!remain) {
				break;
			}
		}
		bufLine = oldBufLine;
		bufPos = oldBufPos;
		setBuffer(bufLine,bufPos);
		if(remain) {
			bufferCustom1[bufInx + 0] = (byte)255;
			bufferCustom1[bufInx + 1] = (byte)255;
			bufferCustom1[bufInx + 2] = (byte)255;
		}
		else {
			bufferCustom1[bufInx + 0] = (byte)0;
			bufferCustom1[bufInx + 1] = (byte)0;
			bufferCustom1[bufInx + 2] = (byte)0;
			//bufferCustom1[bufInx + 0] = bufferBackground[bufInx + 0];
			//bufferCustom1[bufInx + 1] = bufferBackground[bufInx + 1];
			//bufferCustom1[bufInx + 2] = bufferBackground[bufInx + 2];
		}
	}
	public void open(int element) {
		int iMax = getCamera().getFrameSize();
		int jMax = getCamera().getLineSize();
		int oldBufLine = bufLine;
		int oldBufPos = bufPos;
		int newBufLine = bufLine-(element/2);
		int newBufPos = bufPos-(element/2);
		setBuffer(bufLine,bufPos);
		int v1 = bufferDestination[bufInx + 0]&0xFF;
		if(v1==255) {
			setBuffer(newBufLine,newBufPos);
			for (int i = 0; i < element; i++) {
				for (int j = 0; j < element; j++) {
					bufLine = newBufLine+i;
					bufPos = newBufPos+j;
					setBuffer(bufLine,bufPos);
					if(bufPos>=0&&bufPos<bufferDestination.length/(jMax*3)&&bufLine>=0&&bufLine<bufferDestination.length/(iMax*3)) {			
						bufferCustom1[bufInx + 0] = (byte)255;
						bufferCustom1[bufInx + 1] = (byte)255;
						bufferCustom1[bufInx + 2] = (byte)255;
					}
				}
			}
		} else {
			bufferCustom1[bufInx + 0] = (byte)0;
			bufferCustom1[bufInx + 1] = (byte)0;
			bufferCustom1[bufInx + 2] = (byte)0;
			//bufferCustom1[bufInx + 0] = bufferBackground[bufInx + 0];
			//bufferCustom1[bufInx + 1] = bufferBackground[bufInx + 1];
			//bufferCustom1[bufInx + 2] = bufferBackground[bufInx + 2];
			
			
		}
		bufLine = oldBufLine;
		bufPos = oldBufPos;
		setBuffer(bufLine,bufPos);
	}
	
	public void drawLine(int x1b, int y1b, int x2b, int y2b) {
		// this is necessary because bytes are signed in Java
		int x1 = x1b;
		int x2 = x2b;
		int y1 = y1b;
		int y2 = y2b;

		// Change coordinates so that the line is drawn from left to right
		if(x1 > x2) {
			int tmp = x1;
			x1 = x2;
			x2 = tmp;
			
			tmp = y1;
			y1 = y2;
			y2 = tmp;
		}
		
		double stepx = 0; // the size of the step in the x-direction
		double stepy = 0; // the size of the step in the y-direction

		int dx = x2 - x1; // x difference between two pixels
		int dy = y2 - y1; // y difference between two pixels
		
		if (x1 > camera.getLineSize()-1 || x2 > camera.getLineSize()-1 || y1 > camera.getFrameSize()-1 || y2 > camera.getFrameSize()-1 || x1 < 0 || x2 < 0
				|| y1 < 0 || y2 < 0) {
			System.out.println("drawLine : Out of bounds");
			System.out.println("x1 = " + x1 + " y1 = " + y1 + " x2 = " + x2
					+ " y2 = " + y2);
			return;
		}
		
		// cant draw arrow from point x to x
		if(dx == 0 && dy == 0) {
			System.out.println("drawLine : can only draw line between different points");
			return;
		}
		
		// set stepx and stepy		
		if(dy < 0) { 		// up
			if((-1*dy) > dx) {
				stepx = (double)dx / (double)(-1*dy);
				stepy = -1;
			}
			else {
				stepx = 1;
				stepy = (double)dy / (double)dx;
			}
		}
		else if(dy > 0) {  	// down
			if(dy > dx) {
				stepx = (double)dx / (double)dy;
				stepy = 1;
			}
			else {
				stepx = 1;
				stepy = (double)dy / (double)dx;
			}
		}
		else {
			stepx = 1;
			stepy = 0;
		}
		
		int tmpx = x1;
		int tmpy = y1;
		double tmp_x = x1;
		double tmp_y = y1;
		
		boolean horizontal = false;
		boolean vertical = false;
		
		if(dx == 0) {
			horizontal = true;
		}
		if(dy == 0) {
			vertical = true;
		}
		
		// draw the line
		while((tmpx != x2 || horizontal) && (tmpy != y2 || vertical)) {
			// set index
			setBuffer(tmpx,tmpy);
			// write to buffer
			bufferDestination[bufInx + 0] = (byte)255;
			bufferDestination[bufInx + 1] = (byte)0;
			bufferDestination[bufInx + 2] = (byte)0;
			
			// update tmpx and tmpy
			tmp_x = tmp_x + stepx;
			tmp_y = tmp_y + stepy;
			tmpx = (int)tmp_x;
			tmpy = (int)tmp_y;
		}
	}
}