package VM2;

//import javax.microedition.io.StreamConnection;
//import javax.microedition.io.StreamConnectionNotifier;

/**
 * Testprogram UPB<br>
 * This program receives serial data on any JStamp/JStik com port RXD line,
 * and stores the data inside a buffer. <br>
 * <hr>
 * <b>Details:</b><br>
 * An event listener stores up the incoming data and keeps a count. The
 * main line picks up the incoming data and interprets it. The main
 * loop makes sure that the hearbeat led is flahsed and that the
 * incoming data is processed.
 * <hr>
 * <B>Using this program</B>
 * <ol>
 * <li>Change baud rate or com port values in {@link #initializeSerialPort}
 *
 * <li>Be sure you have the right configuration in JemBuilder and you have the
 * correct Project->Drivers installed (com1, com2, com3 as needed).
 *
 * </ol>
 * <hr>
 *
 * @author Peter van Lith
 * @version 0.3
 * <ul>
 * <li>0.3 - 12-01-2004 USB templates version
 * 
 * </ul>
 */
public class JPB
{
	private JPBvm vm;
	
	public JPB()
	{
	}

	/**
	* Instantiate our class then loop in this thread.
	* Reading of incoming data is done in the serial port.
	*/
	public void start(String ipText, String portText, String socket)
	{
		if(vm != null)
		{
			stop(); // Cleanup the old camera and vm
		}
		System.out.println("JCU Test program 0.4");
		
		// Make a localvm or a sockethandler
		if (socket.equals("Socket"))
		{
			vm = new SocketHandler(ipText, portText);
			// TODO set up an appropriate BufO
			/*
			SocketHandler mySocket = new SocketHandler(this, ipText, portText, socket);
			buffer = new byte[mySocket.bufSize()];
			mySocket.setBuffer(buffer);
			mySocket.start();
			LocalJPBvm.setLocal(false);
			System.out.println("Started socket task");
			*/
		}
		else
		{
			vm = new LocalJPBvm(socket);
		}
	}
	
	public void stop()
	{
		if (vm != null) vm.cleanup();
		vm = null;
	}
	
	public JPBvm getVm()
	{
		return vm;
	}
	
	public static String getErrCode(byte[] ret)
	{
		int err;
		err = ret[0] & 0xFF;
		String code;
		int ll;
		switch (err)
		{
		case JPBvm.OK:
			code = "OK";
			break;
			
		case JPBvm.ALIVE:
			code = "Alive";
			break;
			
		case JPBvm.ILL_OPC:
			code = "Illegal opcode";
			break;
			
		case JPBvm.NOT_SUPP:
			code = "Not supported";
			break;
			
		case JPBvm.TIMEOUT:
			code = "Timeout";
			break;
			
		case JPBvm.DATA_OK:
			code = "Data OK";
			break;
			
		case JPBvm.DATA_BAD:
			code = "I/O Error";
			break;
			
		case JPBvm.DATA_1:
			code = "Data: " + toHex(ret[1]);
			break;
			
		case JPBvm.DATA_2:
			code = "Data: " + toHex(ret[1]) + "," + toHex(ret[2]);
			break;
			
		case JPBvm.DATA_X:
			code = "Data: ";
			ll = ret[0] & 0xFF;
			if (ll == JPBvm.DATA_X) code = "Picture:";
			else
			{
				for (int I= 0; I < ll; I++)
				{
					code += toHex(ret[I + 2]);
					if (I < ret[1] - 1) code += ",";
				}
			}
			break;
			
		case JPBvm.READ_FILE:
			code = "Reading file..";
			break;
	
		case JPBvm.WRITE_FILE:
			code = "Writing file..";
			break;
			
		default:
			code = "Unknown error: " + err;
			break;
		}
		return code;
	}
	
	public static String toHex(int aValue)
	{
		String S;
		S = Integer.toHexString(aValue).toUpperCase();
		if (S.length() < 2)
		S = "0" + S;
		return S;
	}
	
	public static String toHex(byte aValue)
	{
		String S;
		int I = aValue & 0xFF;
		return toHex(I);
	}
}