package VM2;

/**
 * <p>
 * Title: UPB Virtual Machine
 * </p>
 * <p>
 * Description: Main Virtual Machine interpreter for UPB
 * </p>
 * <p>
 * Copyright: Copyright (c) 2002-2003 MultiMotions
 * </p>
 * <p>
 * Company: Lithp Systems bv
 * </p>
 * 
 * @author Peter van Lith
 * @version 0.3
 * 
 * Version 3.0 implements a number of buffers for real-time processing. BufO -
 * is the system standard output screen buffer. Please note that this buffer is
 * filled from the local routines. This buffer is first converted to a byte
 * buffer and then later on converted back to a screen buffer. Must be
 * refactored later on bufO - is the output buffer, used to display to the user
 * buf1 - is the input image of the previous frame buf2 - is the input image of
 * the current frame
 * 
 * Processing may only be done in the output buffers. To maintain compatibility,
 * the QCIF buffer is created inside the CIF buffer and therefore uses only a
 * part of the total buffer space.
 */

public class LocalJPBvm implements JPBvm {
	private int opState; // Processor state

	private int opCode; // Stored opCode

	private int opLen; // Length of current instruction

	private int opProp; // Properties of opcode

	private int op1; // Operand 1

	private int op2; // Operand 2

	private int opRet = 0;

	private int opRetCt = 0; // Count of return data

	private boolean bw_color;

	private boolean colorBar = false;

	private boolean cif = true;

	private UserTask aTask;

	private int bufInx;

	private boolean streaming = false;

	private CameraHandler cameraHandler;
	
	private int[] positions;

	public LocalJPBvm(String socket) {
		cameraHandler = new CameraHandler(socket);
	}

	public void reset() {
		opState = 0;
		opLen = 0;
		opProp = 0;
		op1 = 0;
		op2 = 0;
		opRet = 0;
		opRetCt = 0;
	}

	public byte[] executeCommand(byte cmd, String sReg, Object sData) {
		byte theReg = 0;
		byte theData = 0;
		
		if(cmd == 109||cmd == 66){
			positions = (int[])sData;
			System.out.println(positions[3]);
			sData = "0x02";
		}

		int[] theProps;
		int prop;

		try {
			theReg = Byte.decode(sReg).byteValue();
			theData = Byte.decode((String)sData).byteValue();
		} catch (NumberFormatException e) {
		}
		;

		theProps = getProps(cmd);
		prop = theProps[2] & OPC_VIA;

		return Dispatch(cmd, theReg, theData);
	}

	/**
	 * processByte takes the next byte, and depending on the instruction type
	 * collects data before the command is processed. The only exception to this
	 * is the ISYNC command, that will allow resynchronization.
	 */
	private byte[] processByte(int aByte) {
		int[] props;
		byte[] ret = retByte(ACK); // ACK, accept next byte
		int RC;

		switch (opState) {
		case 0: // Process directly
			RC = processProps(aByte);
			if (RC != 0)
				ret = Dispatch(aByte, 0, 0);
			break;

		case 1: // Process after first parm
			if (opLen == 1) {
				ret = Dispatch(opCode, aByte, op2);
				opState = 0;
			}
			if (opLen == 2) {
				op1 = aByte;
				opState = 2;
			}
			if (opLen < 0) {
				opLen = aByte;
				opState = 3;
			}
			break;

		case 2:
			op2 = aByte;
			ret = Dispatch(opCode, op1, op2);
			opState = 0;
			break;

		case 3:
			op2 = aByte;
			opLen--;
			ret = Dispatch(opCode, opLen, op2);
			if (opLen == 0)
				opState = 0;
			else if (ret[0] == OK)
				ret = retByte(ACK);
			break;
		}
		return ret;
	}

	private int processProps(int aByte) {
		int[] props;
		int ret = 0;
		props = getProps(aByte);
		opProp = props[1];
		switch (opProp) {
		case OPC_ZERO: // Check if direct execution
			opState = 0;
			ret = 1;
			break;

		case OPC_ONE:
			opCode = aByte;
			opLen = 1;
			opState = 1;
			break;

		case OPC_TWO:
			opCode = aByte;
			opLen = 2;
			opState = 1;
			break;

		case OPC_VAR:
			opCode = aByte;
			opLen = -1;
			opState = 1;
			break;
		}
		return ret;
	}

	private byte[] interpretCommand(int aCommand)
	// Receives a single command byte
	// Interprets this and executes the command
	// Some commands have a number of parameters
	// They are handled as subsequent bytes
	{
		byte[] ret;
		int[] props;
		int len;
		props = getProps(aCommand);
		ret = Dispatch(aCommand, props[1], op2);
		return ret;
	}

	private int[] getProps(int aCommand) {
		int prop;
		int I;
		int[] props = new int[3];
		for (I = 0; I < CmdTable.length; I = I + 2) {
			if (aCommand == CmdTable[I]) {
				prop = CmdTable[I + 1];
				props[0] = CmdTable[I];
				props[1] = prop & OPC_MASK;
				props[2] = prop & (OPC_REPLY ^ OPC_VIA);
				break;
			}
			props[0] = ILL_OPC; // Generate illigal opcode
			props[1] = 0;
			props[2] = 0;
		}
		return props;
	}

	/**
	 * Dispatch The dispatcher takes a complete command and executes it locally.
	 * Only a limited number of the standard commands are supported.
	 */

	private byte[] Dispatch(int aCommand, int op1, int op2) {
		byte[] ret;
		//System.out.println("command: "+aCommand);
		switch (aCommand) {
		case ISYNC:
			ret = retByte(ALIVE);
			break;
		case VER:
			ret = retByte1(Ver);
			break;
		case SET_CIF: // set CIF
			ret = cmdSET_CIF();
			break;
		case SELF_TEST:
			ret = cmdSELF_TEST(op1);
			break;
		case RESET_JPB:
			ret = cmdRESET_JPB();
			break;
		case READ_REG:
			ret = retByte1(op1);
			break;
		case READ_FRAME: // Read Frame
			ret = cmdREAD_FRAME();
			break;
		case STREAM:
			ret = cmdSTREAM();
			break;
		case READ: // Read next line
			ret = cmdREAD();
			break;
		case STOP:
			ret = cmdSTOP();
			break;
		case COLORBAR: // Set Colorbar
			ret = cmdCOLORBAR();
			break;
		case IMAGE: // Set Image
			ret = cmdIMAGE();
			break;
		case READ_FILE:
			ret = retByte(READ_FILE); // Executed locally in GUI
			break;
		case WRITE_FILE:
			ret = retByte(WRITE_FILE); // Executed locally in GUI
			break;
		case BW_COLOR:
			ret = cmdBW_COLOR();
			break;
		case SET_REG:
			ret = retByte2(op1 * 256 + op2);
			break;
		case ECHO:
			ret = retByte1(op1);
			break;
		case BYE:
			ret = retByte(OK);
			break;
		
		case USR_START:
			ret = cmdUSR_START(op1, op2);
			break;
		case USR_STOP:
			ret = cmdUSR_STOP();
			break;
		case USR_HSI:
			cmdUSR_SINGLE(0x04, 0x00);
			ret = cmdUSR_SETREG(0x04, 0x00);
			break;
		case USR_RGB:
			cmdUSR_SINGLE(0x01, 0x00);
			ret = cmdUSR_SETREG(0x01, 0x00);
			break;
		case USR_NRGB:
			cmdUSR_SINGLE(0x02, 0x00);
			ret = cmdUSR_SETREG(0x02, 0x00);
			break;
		case USR_C1C2C3:
			cmdUSR_SINGLE(0x03, 0x00);
			ret = cmdUSR_SETREG(0x03, 0x00);
			break;
		case USR_I1I2I3:
			cmdUSR_SINGLE(0x11, 0x00);
			ret = cmdUSR_SETREG(0x11, 0x00);
			break;
		case USR_BACKGR_SUB:
			cmdUSR_SINGLE(0x12, 0x00);
			ret = cmdUSR_SETREG(0x12, 0x00);
			break;
		case USR_PEOPLE_COUNT:
			cmdUSR_SINGLE(0x13, op2);
			ret = cmdUSR_SETREG(0x13, op2);
			break;
		case USR_OPTFLOW1:
			cmdUSR_SINGLE(0x17, 0x01);
			ret = cmdUSR_SETREG(0x17, 0x01);
			break;
		case USR_OPENING:
			cmdUSR_SINGLE(0x14, 0x00);
			ret = cmdUSR_SETREG(0x14, 0x00);
			break;
		case USR_CLOSING:
			cmdUSR_SINGLE(0x15, 0x00);
			ret = cmdUSR_SETREG(0x15, 0x00);
			break;
		case USR_PC_HOR:
			cmdUSR_SINGLE(0x16, 0x00);
			ret = cmdUSR_SETREG(0x16, 0x00);
			break;
		case USR_PC_VER:
			// people counting vertical sensor grid
			cmdUSR_SINGLE(0x18, 0x00);
			ret = cmdUSR_SETREG(0x18, 0x00);
			
			break;
		case USR_TEST:
			cmdUSR_SINGLE(0x21, 0x00);
			ret = cmdUSR_SETREG(0x21, 0x00);
			break;
		case USR_BLUR:
			cmdUSR_SINGLE(0x20, 0x00);
			ret = cmdUSR_SETREG(0x20, 0x00);
			break;
		
		
	
		case USR_F13:
			cmdUSR_SINGLE(0x10, 0x00);
			ret = cmdUSR_SETREG(0x10, 0x00);
			break;
		case USR_F14:
			cmdUSR_SINGLE(0x10, 0x00);
			ret = cmdUSR_SETREG(0x10, 0x00);
			break;
		case USR_F15:
			cmdUSR_SINGLE(0x10, 0x00);
			ret = cmdUSR_SETREG(0x10, 0x00);
			break;
		case USR_F16:
			cmdUSR_SINGLE(0x10, 0x00);
			ret = cmdUSR_SETREG(0x10, 0x00);
			break;
		case USR_F17:
			cmdUSR_SINGLE(0x10, 0x00);
			ret = cmdUSR_SETREG(0x10, 0x00);
			break;
		case USR_F21:
			cmdUSR_SINGLE(0x10, 0x00);
			ret = cmdUSR_SETREG(0x10, 0x00);
			break;
		case USR_F22:
			cmdUSR_SINGLE(0x10, 0x00);
			ret = cmdUSR_SETREG(0x10, 0x00);
			break;
		case USR_F31:
			cmdUSR_SINGLE(0x10, 0x00);
			ret = cmdUSR_SETREG(0x10, 0x00);
			break;
		case ILL_OPC:
			ret = retByte(ILL_OPC);
		default:
			ret = retByte(NOT_SUPP);
			break;
		}
		return ret;
	}

	private byte[] cmdRESET_JPB() {
		opCode = 0;
		opLen = 0;
		opRet = 0;
		return retByte(OK);
	}

	private byte[] cmdSET_CIF() {
		if (cif) {
			cif = false;
			return retByte1(1);
		} else {
			cif = true;
			return retByte1(2);
		}
	}

	private byte[] cmdSELF_TEST(int Type) {
		if (Type == 1) // SPI Test
		{
			System.out.println("Executing Self Test - 1");
			return retByte(ALIVE);
		}
		if (Type == 2) // Display Test
		{
			System.out.println("Executing Self Test - 2");
			return retByte(OK);
		}
		if (Type == 3) // IR Test
		{
			System.out.println("Executing Self Test - 3");
			return retByte(OK);
		}
		return retByte(OK);
	}

	private byte[] cmdSELF_TEST_SPI(int Type) {
		int I = 0;
		int RC = 0;
		for (I = 0; I < 32; I++) {
		}
		return retByte(RC);
	}

	private byte[] cmdREAD_FRAME() {
		// Het readen van een frame is nu niets anders, dan ervoor zorgen dat
		// het volgende
		// frame uit de buffer wordt gehaald. Wat we dus teruggeven is alleen
		// een code, die
		// aan de mainForm vertelt of hij aan het streamen is of niet. (zie oude
		// USBCam)

		// TODO andere leesfuncties ook aanpassen

		byte[] ret = new byte[1];

		streaming = false;
		cameraHandler.getCamera().start();

		ret[0] = (byte) JPBvm.OK;
		return ret;
	}

	private byte[] cmdREAD() {
		byte[] ret = new byte[1];
		cameraHandler.getCamera().getNextFrame();
		if (aTask != null) {
			aTask.run();
			//aTask.start();
		}
		if (streaming)
			ret[0] = (byte) JPBvm.DATA_OK;
		else
			ret[0] = (byte) JPBvm.OK;
		return ret;
	}

	private byte[] cmdSTREAM() {
		byte[] ret = new byte[1];

		streaming = true;
		cameraHandler.getCamera().start();

		ret[0] = (byte) JPBvm.DATA_OK;
		return ret;
	}

	private byte[] cmdSTOP() {
		streaming = false;
		cameraHandler.getCamera().stop();
		return retByte(OK);
	}

	private byte[] cmdCOLORBAR() {
		streaming = false;
		return cameraHandler.getBuffer();
	}

	private byte[] cmdIMAGE() {
		streaming = false;
		return cameraHandler.getBuffer();
	}

	private byte[] cmdBW_COLOR() {
		if (bw_color)
			bw_color = false;
		else
			bw_color = true;

		return retByte(OK);
	}

	private byte[] cmdSET_REG(int Port, int Data) {
		int ret = 0;
		return retByte(ret);
	}

	private byte[] cmdREAD_REG(int Port) {
		int ret = 0;
		return retByte(ret);
	}

	private byte[] cmdUSR_SINGLE(int op1, int op2) {
		try {
			if (aTask == null) {
				aTask = new UserTask(cameraHandler, true, // single task
						op1, op2, positions);

				aTask.run();
				//aTask.start();
				aTask = null;
			}
			return retByte(OK);
		} catch (Exception e) {
			System.out.println("Error during single: " + e);
			e.printStackTrace();
			return retByte(TIMEOUT);
		}
	}
	
	private byte[] cmdUSR_START(int op1, int op2) {
		if (aTask == null) {
			aTask = new UserTask(cameraHandler, false, // not a single task
					op1, op2, positions);
			aTask.run();
			//aTask.start();
			return retByte(OK);
		}
		return retByte(TIMEOUT);
	}

	private byte[] cmdUSR_STOP() {
		try {
			if (aTask != null) {
				aTask.end();
				aTask = null;
			}
			return retByte(OK);
		} catch (Exception e) {
			System.out.println("Error during stop: " + e);
			return retByte(TIMEOUT);
		}
	}

	private byte[] cmdUSR_SETREG(int User1, int User2) {
		return retByte2(User1, User2);
	}

	private byte[] NYet(int aCommand) {
		return retByte(NOT_SUPP);
	}

	private byte[] retByte(int aValue) {
		byte[] ret = new byte[1];
		ret[0] = (byte) aValue;
		return ret;
	}

	private byte[] retByte(byte aValue) {
		byte[] ret = new byte[1];
		ret[0] = aValue;
		return ret;
	}

	private byte[] retByte1(int aValue) {
		byte[] ret = new byte[2];
		ret[0] = (byte) DATA_1;
		ret[1] = (byte) aValue;
		return ret;
	}

	private byte[] retByte2(int aValue) {
		byte[] ret = new byte[3];
		ret[0] = (byte) DATA_2;
		ret[1] = (byte) (aValue / 256);
		int I = aValue - (ret[1] * 256);
		ret[2] = (byte) I;
		return ret;
	}

	private byte[] retByte2(int Val1, int Val2) {
		byte[] ret = new byte[3];
		ret[0] = (byte) DATA_2;
		ret[1] = (byte) Val1;
		ret[2] = (byte) Val2;
		return ret;
	}

	private byte[] retByteX(int aValue) {
		int I;
		byte[] ret = new byte[2];
		ret[0] = (byte) DATA_X;
		ret[1] = (byte) aValue;
		return ret;
	}

	private byte[] retByteX(byte[] aValue) {
		int I;
		byte[] ret = new byte[aValue.length + 1];
		ret[0] = (byte) DATA_X;
		for (I = 0; I < aValue.length; I++) {
			ret[I + 1] = aValue[I];
		}
		return ret;
	}

	private byte[] retInt(int aValue) {
		byte[] ret = new byte[2];
		ret[0] = (byte) (aValue / 256);
		int I = aValue - (ret[0] * 256);
		ret[1] = (byte) I;
		return ret;
	}

	public int getFrameSize() {
		return cameraHandler.getCamera().getFrameSize();
	}

	public int getLineSize() {
		return cameraHandler.getCamera().getLineSize();
	}

	public CameraHandler getCamera() {
		return cameraHandler;
	}

	public byte[] getBuffer() {
		return cameraHandler.getBuffer();
	}

	public void cleanup() {
		if (aTask != null)
			aTask.end();
		aTask = null;
		cameraHandler.cleanup();
		cameraHandler = null;
	}
}