package VM2;

/**
 * <p>Title: JPB virtual machine</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Lithp Systems bv</p>
 * @author Peter van Lith
 * @version 1.0
 * 
 * The OV6620 has a Bayer layout of pixels in the following grid:
 *     
 *    	BGBG....BGBG
 * 		GRGR....GRGR
 * 		BGBG....BGBG
 * 		GRGR....GRGR
 * 		BGBG....BGBG
 * 
 * The various modes send data in the following formats. Each
 * of these formats sends the data in a redundant way. For
 * each format the data stream is different:
 * - RGB   - Raw RGB format
 *           BGRG....BGRG
 * - YUV   - YUV format
 *           UYVY....UYVY
 * - YCrCb - format	
 * 			 BYRY....BYRY
 */

public class ScreenBuffer
{
  private JPB myJPB;                   // JPB
  private int lines;
  private static LineBuffer[] Buffer;
  public int frameSize;
  public int lineSize;
  public boolean bw_color=false;

  /**
  * Sets up the state of a new socket
  * @param theJPB JPB
  */
  public ScreenBuffer(JPB theJPB, int X, int Y)
  {
    int I;
    myJPB = theJPB;
    frameSize = Y;
    lineSize = X;
    Buffer = new LineBuffer[lineSize*3+3];
    lines = 0;
  }
  
  public ScreenBuffer(int X, int Y)
  {
 	frameSize = Y;
  	lineSize = X;
  	Buffer = new LineBuffer[lineSize*3+3];
  	lines = 0;
  }

   public String makeHeader()
	{
		int I;
		String S="    ";
		String X;
		for (I= 0; I < lineSize; I=I+4)
//		for (I= 0; I < JPBvm.lineLength; I=I+4)
//		for (I= 0; I < 28; I=I+4)
		{
			X= I + "--------";
			S += X.substring(0, 8);
			S += " ";
		}
		S += "\n";
		return S;
	}

	public String toString()
	{
		int I;
		String frameBuf= makeHeader();
		for (I=1; I<lines; I++)
			frameBuf += LineBuffer.setLineHex(Buffer[I]) + "\n";
		return frameBuf;
	}
	
   public byte[] BayerToPixels(byte[] pixels)
   {
        if (bw_color)
   		    BayerToPixelsBW(pixels);      
   		else
   		    BayerToPixelsBG(pixels); 
   	    return pixels;  
   }
   
	public void BayerToPixelsBW(byte[] pixels)
	{
		int I, J;
		LineBuffer aLine;
		int lineNo;
		int lineInx;
		int index;
			byte nil = (byte) 0;
		for (I= 1; I < lines; I++)
		{
			aLine= Buffer[I];
			lineNo= frameSize - aLine.getLine() - 1;
			lineInx= lineNo * lineSize * 3;
			for (J= 0; J < lineSize; J= J + 2)
			{
				index = lineInx + J * 3;
				if ((lineNo & 1) == 0)    // Even line GR
				{
//					if (J < 172)
					if (J < 352)
					{
						pixels[index + 0]= aLine.getBuffer()[J + 3]; // Green
						pixels[index + 1]= aLine.getBuffer()[J + 3];
						pixels[index + 2]= aLine.getBuffer()[J + 3];
					    
						pixels[index + 3]= aLine.getBuffer()[J + 4]; // Red
						pixels[index + 4]= aLine.getBuffer()[J + 4];
						pixels[index + 5]= aLine.getBuffer()[J + 4];
					}
				}
				else                      // Odd line GR
				{
//					if (J < 172)
					if (J < 352)
					{
						pixels[index + 0]= aLine.getBuffer()[J + 3]; // Blue
						pixels[index + 1]= aLine.getBuffer()[J + 3];
						pixels[index + 2]= aLine.getBuffer()[J + 3];
					    
						pixels[index + 3]= aLine.getBuffer()[J + 4]; // Green
						pixels[index + 4]= aLine.getBuffer()[J + 4];
						pixels[index + 5]= aLine.getBuffer()[J + 4];
					}

				}
			}
		}
	}
	
	public void BayerToPixelsBG(byte[] pixels)
	{
		int I, J;
		LineBuffer aLine;
		int lineNo;
		int lineInx;
		int index;
		byte R, G, B;
		byte nil = (byte) 0;
		for (I= 1; I < lines; I++)
		{
			aLine= Buffer[I];
			lineNo= frameSize - aLine.getLine();
			lineInx= lineNo * lineSize * 3;
			for (J= 0; J < lineSize; J= J + 2)
			{
				index= lineInx + J * 3;
				if ((lineNo & 1) == 0) // Even line GR
				{
					//						if (J < 172)
					if (J < 352)
					{
						G= aLine.getBuffer()[J + 3];
						R= aLine.getBuffer()[J + 4];
						pixels[index + 0]= nil; // Green
						pixels[index + 1]= G;
						pixels[index + 2]= nil;
						pixels[index + 3]= R;   // Red
						pixels[index + 4]= nil;
						pixels[index + 5]= nil;
					}
				}
				else // Odd line GR
					{
					//					if (J < 172)
					if (J < 352)
					{
						B= aLine.getBuffer()[J + 3];
						G= aLine.getBuffer()[J + 4];
						pixels[index + 0]= nil; // Blue
						pixels[index + 1]= nil;
						pixels[index + 2]= B;
						pixels[index + 3]= nil; // Green
						pixels[index + 4]= G;
						pixels[index + 5]= nil;
					}
				}
			}
		}
	}
	
	public void GRGBtoPixels(byte[] pixels)
	{
		int I, J;
		LineBuffer aLine;
		int lineNo;
		int index;
		for (I= 0; I<lines; I++)
		{
			aLine= Buffer[I];
			lineNo= aLine.getLine();
			index= lineNo * lineSize * 12;
			for (J= 0; J < lineSize; J=J+4)
			{
				pixels[index + 0]= 0;                           // G
				pixels[index + 1]= aLine.getBuffer()[J + 2];    
				pixels[index + 2]= 0;    
				pixels[index + 3]= aLine.getBuffer()[J + 3];         // R
				pixels[index + 4]= 0;    
				pixels[index + 5]= 0;    
				pixels[index + 6]= 0;                           // G
				pixels[index + 7]= aLine.getBuffer()[J + 4];    
				pixels[index + 8]= 0;    
				pixels[index + 6]= 0;                           // B
				pixels[index + 7]= 0;    
				pixels[index + 8]= aLine.getBuffer()[J + 5];    
			}
		}
	}
	
	public byte[] USBtoPixels(byte[] pixels)
	{
		int I, J, K, X;
		LineBuffer aLine;
		int lineNo;
		int index;
		X = lineSize * 3;
		for (I= 0; I < lines; I++)
		{
			aLine= Buffer[I];
			lineNo= lines - aLine.getLine();
			index= lineNo * X;
			for (J= 0; J < X; J=J+3)
			{
				K= index + J;
				pixels[K]= aLine.getBuffer()[J+2];     // R
				pixels[K + 1]= aLine.getBuffer()[J+3]; // G
				pixels[K + 2]= aLine.getBuffer()[J+4]; // B
			}
		}
		return pixels;
	}
	
	public void RGBtoPixels(byte[] pixels)
	{
		int I, J, K;
		LineBuffer aLine;
		int lineNo;
		int index;
		for (I= 0; I < lines; I++)
		{
			aLine= Buffer[I];
			lineNo= aLine.getLine();
			index= lineNo * lineSize * 3;
			for (J= 0; J < lineSize; J=J+3)
			{
				K = index + J*3;
				pixels[K + 0]= aLine.getBuffer()[J + 2];        // R
				pixels[K + 1]= 0;
				pixels[K + 2]= 0;
				if (J < 174)
				{
					pixels[K + 3]= 0;                      // G
					pixels[K + 4]= aLine.getBuffer()[J + 3];
					pixels[K + 5]= 0;
					pixels[K + 6]= 0;                      // B
					pixels[K + 7]= 0;
					pixels[K + 8]= aLine.getBuffer()[J + 4];
				}
			}
		}
	}
	
	public void YUVtoPixels(byte[] pixels)
	{
		int I, J;
		LineBuffer aLine;
		int lineNo;
		int index;
		for (I= 0; I<lines; I++)
		{
			aLine= Buffer[I];
			lineNo= aLine.getLine();
			index= lineNo * lineSize * 12;
			for (J= 0; J < lineSize; J=J+4)
			{
				pixels[index + 0]= 0;                           // G
				pixels[index + 1]= aLine.getBuffer()[J + 2];    
				pixels[index + 2]= 0;    
				pixels[index + 3]= aLine.getBuffer()[J + 3];         // R
				pixels[index + 4]= 0;    
				pixels[index + 5]= 0;    
				pixels[index + 6]= 0;                           // G
				pixels[index + 7]= aLine.getBuffer()[J + 4];    
				pixels[index + 8]= 0;    
				pixels[index + 6]= 0;                           // B
				pixels[index + 7]= 0;    
				pixels[index + 8]= aLine.getBuffer()[J + 5];    
			}
		}
	}

	public void addLine(LineBuffer aLine)
	{
		int lineNo= aLine.getLine();
		if (lineNo == 1) // If first line
		{
			if (Buffer[0] == null)
				Buffer[0]= LineBuffer.makeLine(0);
		}
		Buffer[lineNo]= aLine;
		lines= lineNo; // Remember number of lines
	}
	
	public LineBuffer getLine(int aLine)	
	{
		return Buffer[aLine];
	}

 }
