package VM2;
/*
 * @author Derk Crezee & Daan van Schuppen (UVA)
 * 
 * The Sensor class is able to detect motion on a certain location of a 
 * particular width and height on the sreen based on a sensing function
 */
public class Sensor
{
	private CameraHandler myCameraHandler;
	private int blockSize = 3;
	private int stepSize = 1;
	private int xpos;
	private int ypos;
	private int width;
	private int height;
	private boolean initialized = false;
	private String name = "";
	
	private boolean active = true;
	private int inactiveTime = 0;
	private int currentTime = 0;
	private double threshold;
	
	

	/* 
	 * - myCameraHandler: CameraHandler
	 * - xpos: x position of the center of the sensor
	 * - ypos: y position of the center of the sensor
	 * - width: width of the sensor
	 * - height: height of the sensor
	 * - name: name of the sensor
	 * - blocksize: 
	 * - stepsize: the number of pixels skipped when placing sensor pixels
	 * - time: the time that a sensor is inactive after exceeding the threshold
	 * - threshold: the percentage of pixels that must be active to let the sensor
	 *   become inactive
	 */
	public Sensor(CameraHandler camHandler, int x, int y, int w, int h, String name,int blocksize, int stepsize, int time,double threshold) {
		this.myCameraHandler = camHandler;
		this.blockSize = blocksize;
		this.stepSize = stepsize;
		this.xpos = x;
		this.ypos = y;
		this.width = w;
		this.height = h;
		this.name = name;
		this.inactiveTime = time;
		this.threshold = threshold;
		initialize();
	}
	public Sensor(CameraHandler camHandler, int xpos, int ypos, int width, int height, String name, int time, double threshold) {
		this.myCameraHandler = camHandler;
		this.xpos = xpos;
		this.ypos = ypos;
		this.width = width;
		this.height = height;
		this.name = name;
		this.inactiveTime = time;
		this.threshold = threshold;
		initialize();
	}

	
	/*
	 * initialize: initalizes the sensor, it checks whether sensor fits in the screen
	 * 
	 */
	public void initialize() {
		int maxW = myCameraHandler.getCamera().getLineSize() - Math.round(blockSize/2);
		int maxH = myCameraHandler.getCamera().getFrameSize() - Math.round(blockSize/2);
		
		if ( ( this.xpos + Math.round(this.width/2) < maxW) && ( this.ypos + Math.round(this.height/2) < maxH) &&
				(this. xpos - Math.round(this.width/2) > 0) && ( this.ypos - Math.round(this.height/2) > 0)) {
			this.initialized = true;	// sensor is initialized			
		} else {
			this.initialized = false;	// sensor is not initialized, sensor value will always be 0
		}	
		
	}
	/*
	 *  setName: sets the name of the sensor
	 */
	public void setName(String name) {
		this.name = name;	
	}
	/*
	 * getName: returns the name of the sensor 
	 */
	public String getName() {
		return this.name;
	}
	/*
	 * sense: returns the percentage of the number of pixels that are active
	 * 		  in the sensor
	 */
	public double sense() {
		int totalPixels = 0;
		int activePixels = 0;
		
		
		// if initialized, else return value will always be 0
		if (this.initialized) {
			
			// if the sensor is currently active
			if (this.active) {		
				int iMin = this.ypos - Math.round(this.height/2);
				int iMax = this.ypos + Math.round(this.height/2);
				
			
				int jMin = this.xpos - Math.round(this.width/2);
				int jMax = this.xpos + Math.round(this.width/2);
				
				// loop through the pixels in the sensor with steps stepSize
				for (int i=iMin; i<iMax; i=i+this.stepSize)
				{
					myCameraHandler.setY(i);
					for (int j=jMin; j<jMax; j=j+this.stepSize)
					{
						myCameraHandler.setX(j);
						
						// if the sense function in myCameraHandler returns true
						// the current pixel will be active
						if (myCameraHandler.sense(this.blockSize,1)) {
							activePixels++;	
						} 	
						totalPixels++;
					}
				
				}
			} else {
				// sensor is inactive while currentTime < inactiveTime
				totalPixels=0;
				activePixels=0;
				currentTime++;
				if (currentTime>=inactiveTime) {
					currentTime = 0;
					this.active = true;
				}
			}
		} 
		
		if (totalPixels!=0) {
			
			// percentage of active pixels
			double activation = (Math.floor((((double)activePixels/(double)totalPixels)*100)))/100;
			
			// the sensor will become inactive
			if (activation > threshold) {
				this.currentTime = 0;
				this.active = false;
			}
			return activation;	
		} else {
			return 0;	
		}
		
	}
	/*
	 * setInactive: sets the sensor inactive
	 */
	public void setInactive() {
		this.active = false;
	
	}
	/*
	 * getStatus: returns true if the sensor is active, otherwise false
	 */
	public boolean getStatus() {
		return this.active;
	}
	
	// TODO: somehow this functions does not work
	public void showSensor(int color) {
		if (this.initialized) {
					
			int iMin = this.ypos - Math.round(this.height/2);
			int iMax = this.ypos + Math.round(this.height/2);
			
		
			int jMin = this.xpos - Math.round(this.width/2);
			int jMax = this.xpos + Math.round(this.width/2);
			
			
			for (int i=iMin; i<iMax; i=i++)
			{
				myCameraHandler.setBuffer(jMin,i);
				myCameraHandler.fillPixel(1,color); 	
			}
				
			
			for (int i=iMin; i<iMax; i=i++)
			{
				myCameraHandler.setBuffer(jMax,i);
				myCameraHandler.fillPixel(1,color); 	
			
			}
			myCameraHandler.setY(iMin);
			for (int j=jMin; j<jMax; j=j++)
			{
				myCameraHandler.setBuffer(j,iMin);
				myCameraHandler.fillPixel(1,color); 	
			}
			
			
			for (int j=jMin; j<jMax; j=j++)
			{
				myCameraHandler.setBuffer(j,iMax);
				myCameraHandler.fillPixel(1,color); 	
			}
		} 
	
	
		
	}
}