package VM2;

import java.util.*;

public class UserTask implements Runnable
//extends Thread
{
	private long startMillis = 0;
	private long elapseMillis = 0;
	private long timeStamp = 0;
	private int count;
	private CameraHandler myCameraHandler;
	private int lineSize;
	private int frameSize;
	private boolean running;
	private boolean finished;
	private boolean single;
	private int op1 = 0;
	private int op2 = 0;
	private byte R, G, B;
	
	// added 2005
	private int[] positions = {-1,-1,-1,-1,-1,-1,-1,-1};
	private int currentColorModel=0;
	private int colorModelThreshold = 0;
	private int backgroundSubtractionThreshold = 20;
	
	private boolean backgroundInitialized = false;
	private int backgroundInitializationCounter = 0;
	private int averageNumberOfFrames = 20;
	
	private regionMotions motions;
	private doorwayCounting doorway;
	private boolean sensorInit = false;
	
	private sensorGrid grid1,grid2;
	

	public UserTask(CameraHandler theCamera, boolean single, int op1, int op2, int[] positions) {
		myCameraHandler = theCamera;
		lineSize = myCameraHandler.getCamera().getLineSize();
		frameSize = myCameraHandler.getCamera().getFrameSize();
		this.single = single;
		this.op1 = op1;
		this.op2 = op2;
		running = false;
		finished = true; // the task can be removed if it's not running yet
		this.positions = positions;
		if (positions!=null ){
			System.out.println("Pos: " + positions[3]);
		}
	}

	// Make run() synchronized so that it will set running and finished
	// properly before end() can be called
	public synchronized void run() {
		String op = JPB.toHex(op1);
		//System.out.println("User task 0x" + op + " started");
		//System.out.println("op2: " + op2);
		running = true;
		finished = false;
		try {
			ProcessUSM(op1, op2);
			wait(1);
			
			//System.out.println("User task 0x" + op + " stopped");
		}
		/*
		 * // oude try, inclusief timen overigens (in commentaar) try { while
		 * (running) {
		 * 
		 * //startMillis = System.currentTimeMillis(); //for(int i=0; i < 100;
		 * i++) { // Process User State Machines ProcessUSM(op1, op2);
		 * 
		 * //}
		 * 
		 * //timeStamp = System.currentTimeMillis(); //elapseMillis = timeStamp -
		 * startMillis;
		 * 
		 * //System.out.println("running time (ms): " + elapseMillis / 100.0);
		 * 
		 * 
		 * if (elapseMillis >= 1000) // Wait for remainder of cycle {
		 * //startMillis = System.currentTimeMillis(); // Restart the clock if
		 * (!single) System.out.println("Count=" + count); count = 0; } wait(1);
		 * if (single) running = false; } System.out.println("User task "+op1+"
		 * stopped"); }
		 */
		catch (Exception e) {
			System.out.print("Caught exception ");
			e.printStackTrace();
			System.out.print(" when processing state machines");
			System.out.println();
		}

		finished = true; // The task can be safely removed now
	}

	public synchronized void end() {
		running = false;
		try {
			while (!finished)
				wait();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	//Timing table: PC Jstik est
	// 0 - dummy - 1320000 70000 = 18x
	// 1 - interpolate - 55 3
	// 2 - reduce - 65 3.6
	// 3 - line - 7000 388
	// 4 - column - 7500 416
	// 5 - 1x1 - 350000
	// 6 - 2x2 - 335000 33333
	// 7 - 3x3 - 160000 13888
	// 8 - 4x4 - 160000
	// 8 - 8x8
	// 9 - 9x9 - 35000 2777
	//10 - 16x16 - 3900 83
	//11 - 32x32 - 4500
	//12 - 64x64 - 1500 72
	//13 - 128x128 - 400 22
	//14 - 256x256 - 100 5.5

	private void ProcessUSM(int op1, int op2) {
		//timeStamp= System.currentTimeMillis();
		//elapseMillis= timeStamp - startMillis;
		switch (op1) {
		case 0x00:
			break;
		case 0x01:
			// standard RGB color model
			currentColorModel = -1;
			colorModelThreshold = 0;
			backgroundSubtractionThreshold = 20;
			
			interpolate();
			break;
		case 0x02:
			// normalized RGB color model
			currentColorModel = 0;
			colorModelThreshold = 20;
			backgroundSubtractionThreshold = 2;
			
			colorModel(0);
			
			break;
		case 0x03:
			// c1c2c3 color model
			colorModelThreshold = 10;
			currentColorModel = 1;
			backgroundSubtractionThreshold = 20;
			
			colorModel(1);
			break;
		case 0x04:
			// HSI color model
			colorModelThreshold = 30;
			currentColorModel = 3;
			backgroundSubtractionThreshold = 20;
			
			colorModel(3);
			break;
		case 0x05:
			processBlock(1);
			break;
		case 0x06:
			processBlock(2);
			break;
		case 0x07:
			processBlock(3);
			break;
		case 0x08:
			processBlock(4);
			break;
		case 0x09:
			processBlock(8);
			break;
		case 0x0A:
			processBlock(9);
			break;
		case 0x0B:
			processBlock(16);
		case 0x0C:
			processBlock(32);
			break;
		case 0x0D:
			processBlock(64);
			break;
		case 0x0E:
			processBlock(128);
			break;
		case 0x0F:
			processBlock(256);
			break;
		case 0x10:
			processFilter();
			break;
		case 0x11:
			// l1l2l3 color model
			colorModelThreshold = 20;
			currentColorModel = 2;
			backgroundSubtractionThreshold = 20;
			
			colorModel(2);
			break;
		case 0x12:
			// background subtraction
			currentColorModel = 0;
			colorModelThreshold = 20;
			backgroundSubtractionThreshold = 1;
			// use the first frames to build up an average background image
			if (backgroundInitializationCounter < averageNumberOfFrames) {
				myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
				myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_BACKGR);
				int I;
				int J;
				for (I= 0; I < frameSize; I++)
				{
					for (J= 0; J < lineSize; J++)
					{
						myCameraHandler.setBuffer(J,I);
						//useColorModel(currentColorModel, colorModelThreshold);
						//useColorModel(0,20);
						myCameraHandler.addToBackground(backgroundInitializationCounter);
					}
				}
				backgroundInitializationCounter++;
				backgroundInitialized=true;
				
			} else {
				subtractBackground();
			}
			
			break;
		case 0x13:
			// people count
			if (!sensorInit) {
				this.doorway = new doorwayCounting(myCameraHandler);
				System.out.println("Sensor initialized");
				sensorInit = true;
			}
			motionSensor();
			break;
		case 0x14:
			// background subtraction followed by closing and opening
			
			// use the first frames to build up an average background image
			if (backgroundInitializationCounter < averageNumberOfFrames) {
				
				myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
				myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_BACKGR);
				int I;
				int J;
				
				for (I= 0; I < frameSize; I++)
				{
					for (J= 0; J < lineSize; J++)
					{
						myCameraHandler.setBuffer(J,I);
						//useColorModel(currentColorModel, colorModelThreshold);
						//useColorModel(0,20);
						myCameraHandler.addToBackground(backgroundInitializationCounter);
						

					}
				}
				backgroundInitializationCounter++;
				if (backgroundInitializationCounter==averageNumberOfFrames) {
					motions = new regionMotions(myCameraHandler,600,50,regionMotions.CORNER_9X9);
					
				}
				backgroundInitialized=true;
				
			} else {
				subtractBackground();
				// close with an n by n stamp
				close(3);
				// open withn an n by n stamp
				open(15);
			}
			break;
			
		case 0x15:
			// background subtraction followed by a closing
			
			// use the first frames to build up an average background image
			if (backgroundInitializationCounter < averageNumberOfFrames) {
				myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
				myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_BACKGR);
				int I;
				int J;
				
				for (I= 0; I < frameSize; I++)
				{
					for (J= 0; J < lineSize; J++)
					{
						myCameraHandler.setBuffer(J,I);
						//useColorModel(currentColorModel, colorModelThreshold);
						//useColorModel(0,20);
						myCameraHandler.addToBackground(backgroundInitializationCounter);
						

					}
				}
				backgroundInitializationCounter++;
				if (backgroundInitializationCounter==averageNumberOfFrames) {
					motions = new regionMotions(myCameraHandler,300,50,regionMotions.CORNER_9X9);
					
				}
				backgroundInitialized=true;
				
			} else {
				subtractBackground();
				// close with a n by n stamp
				close(3);
			}
			break;
		case 0x16:
			// people count using two horizontal grid sensors
			if (!sensorInit) {
				
				this.doorway = new doorwayCounting(myCameraHandler, positions,true);
				System.out.println("Sensor initialized");
				sensorInit = true;
			}
			motionSensor();
			
			break;
		case 0x17:
			// optical flow based on a grid sensor
			if (!sensorInit) {
				// initialization of the grid
				int frameHeight = myCameraHandler.getCamera().getFrameSize();
				int lineSize = myCameraHandler.getCamera().getLineSize();
				grid1 = new sensorGrid(myCameraHandler,10,frameHeight/2,lineSize-10,frameHeight-10,"ms1",1,6,4,3,0.3,true,false, 1);
				grid2 = new sensorGrid(myCameraHandler,lineSize/2,10,lineSize-10,frameHeight-10,"ms1",1,6,4,3,0.3,false,true, 1);
				System.out.println("Sensor initialized");
				sensorInit = true;
			} else {
				gridSensor();
			}
			break;
		case 0x18:
			
			break;
		case 0x20:
			// optical flow based on 
			currentColorModel = 0;
			colorModelThreshold = 20;
			backgroundSubtractionThreshold = 2;
			// use the first frames to build up an average background image
			if (backgroundInitializationCounter < averageNumberOfFrames) {
				
				myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
				myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_BACKGR);
				int I;
				int J;
				
				for (I= 0; I < frameSize; I++)
				{
					for (J= 0; J < lineSize; J++)
					{
						myCameraHandler.setBuffer(J,I);
						//useColorModel(currentColorModel, colorModelThreshold);
						//useColorModel(0,20);
						myCameraHandler.addToBackground(backgroundInitializationCounter);
						

					}
				}
				backgroundInitializationCounter++;
				if (backgroundInitializationCounter==averageNumberOfFrames) {
					motions = new regionMotions(myCameraHandler,600,80,regionMotions.CORNER_3X3);
					
				}
				backgroundInitialized=true;
				
			} else {
				subtractBackground();
				close(3);
				open(15);
				boundingBox();
			}
			break;
		case 0x21:
			//  people count using one vertical grid sensor
			if (!sensorInit) {
				this.doorway = new doorwayCounting(myCameraHandler, positions,false);
				System.out.println("Sensor initialized");
				sensorInit = true;
			}
			motionSensor();
			
			break;
		case 0x19:
			break;
		default:
			System.out.println("Code: " + op1 + " unknown");
			break;
		}
		count++;
	}

	// Interpolate takes a block of 4 pixels on an even and odd line
	// and combines them into four identical pixels with the combined
	// color. It outputs into the pixel buffer which is updated when
	// ready.
	// The format is supposed to be as follows:
	// - XGXRXX
	// - XXBXGX
	// This is converted to the average RGB for the block and all
	// four pixels are set to the same color.
	private void interpolate() {
		int I;
		int J;
		byte[] RGB = new byte[3];

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);

		for (I = 0; I < frameSize; I += 2) {
			for (J = 0; J < lineSize; J += 2) {
				myCameraHandler.getBufferRGB(J, I, RGB);

				myCameraHandler.setBufferRGB(J, I, RGB);
				myCameraHandler.setBufferRGB(J + 1, I, RGB);
				myCameraHandler.setBufferRGB(J, I + 1, RGB);
				myCameraHandler.setBufferRGB(J + 1, I + 1, RGB);
			}
		}
	}

	// Reduce takes the picture and reduces the 4 pixels per group
	// into a single pixel and clears the remainder.
	private void reduce() {
		int I, J;
		byte[] RGB = new byte[3];
		byte[] black = new byte[] { 0, 0, 0 };

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);

		for (I = 0; I < frameSize; I++) {
			for (J = 0; J < lineSize; J++) {
				if (I < frameSize / 2 && J < lineSize / 2) {
					myCameraHandler.getBufferRGB(J * 2, I * 2, RGB);
					myCameraHandler.setBufferRGB(J, I, RGB);
				} else {
					myCameraHandler.setBuffer(J, I);
					myCameraHandler.setBufferRGB(black);
				}
			}
		}
	}

	// The following functions are mostly for test purposes
	// They perform a calculation of the dominant color
	// to check the performance of the system
	private byte[] processBlock(int X) {
		int I;
		int J;
		byte[] RGB = new byte[3];
		byte[] color;

		clearDomColor();
		for (I = 0; I < X; I += 2) {
			myCameraHandler.setX(I);
			for (J = 0; J < X; J += 2) {
				myCameraHandler.setY(J);
				myCameraHandler.getBufferRGB(RGB);
				setDomColor(RGB);
			}
		}
		return getDomColor();
	}

	private void processLine(int aLine) {
		int I;
		int J;
		byte[] RGB = new byte[3];
		byte[] color;

		clearDomColor();
		myCameraHandler.setX(aLine);
		for (J = 0; J < lineSize; J += 2) {
			myCameraHandler.setY(J);
			myCameraHandler.getBufferRGB(RGB);
			setDomColor(RGB);
		}
	}

	private void processColumn(int aCol) {
		int I;
		int J;
		byte[] RGB = new byte[3];
		byte[] color;

		clearDomColor();
		for (I = 0; I < frameSize; I += 2) {
			myCameraHandler.setX(I);
			myCameraHandler.setY(aCol);
			myCameraHandler.getBufferRGB(RGB);
			setDomColor(RGB);
		}
	}

	private void processFilter() {
	}

	
	private void opticalFlow() {
		int i, iMin, iMax, j, jMin, jMax;
		byte[] dimensions = new byte[2];

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		myCameraHandler.bufferCopy();

		dimensions[0] = (byte) 3;
		dimensions[1] = (byte) 3;

		iMin = (int) (dimensions[1] / 2);
		iMax = myCameraHandler.getCamera().getFrameSize() - dimensions[1]
				+ iMin;

		jMin = (int) (dimensions[0] / 2);
		jMax = myCameraHandler.getCamera().getLineSize() - dimensions[0] + jMin;

		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		myCameraHandler.setScratchBuffer(CameraHandler.BUFFERID_CUSTOM1);

		// loop de image door. (0,0) ligt in de rechter onderhoek
		// Apply convolution with the selected matrix on buffer 2, put the
		// result in buffer 1

		if (op2 == 0) {
			for (i = iMin; i <= iMax; i++) {
				myCameraHandler.setY(i);
				for (j = jMin; j <= jMax; j++) {
					myCameraHandler.setX(j);
					myCameraHandler.optFlow();
				}
			}
		}
		if (op2 == 1) {
			for (i = iMin; i <= iMax; i += 10) {
				myCameraHandler.setY(i);
				for (j = jMin; j <= jMax; j += 10) {
					myCameraHandler.setX(j);
					myCameraHandler.drawOptFlow();
				}
			}
		}

		// back up previous image
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.bufferCopy();
	}

	private void processMotion() {
		int i, iMin, iMax, j, jMin, jMax;
		int blockSize = 4;

		iMin = Math.round(blockSize / 2);
		iMax = myCameraHandler.getCamera().getFrameSize()
				- Math.round(blockSize / 2);

		jMin = iMin;
		jMax = myCameraHandler.getCamera().getLineSize()
				- Math.round(blockSize / 2);

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setScratchBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);

		// loop de image door. (0,0) ligt in de linker onderhoek
		for (i = iMin; i < iMax; i++) {
			myCameraHandler.setY(i);
			for (j = jMin; j < jMax; j++) {
				myCameraHandler.setX(j);
				myCameraHandler.motion(blockSize);
			}
		}

		// back up previous image
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.bufferCopy();
	}

	/*
	 * This method calculates the first derivative of an image using the Sobel
	 * masks by calling the method firstDerivative in CameraHandler. op2 is used
	 * as the threshold parameter.
	 */
	private void firstDerivative() {
		byte[] dimensions;

		// we need the dimensions, FIRST_X is the same as FIRST_Y, since they
		// are
		// squared
		dimensions = myCameraHandler
				.setConvolutionMatrix(CameraHandler.FIRST_X_3X3);

		int iMin = (int) (dimensions[1] / 2);
		int iMax = myCameraHandler.getCamera().getFrameSize() - dimensions[1]
				+ iMin;

		int jMin = (int) (dimensions[0] / 2);
		int jMax = myCameraHandler.getCamera().getLineSize() - dimensions[0]
				+ jMin;

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);

		for (int i = iMin; i <= iMax; i++) {
			myCameraHandler.setY(i);
			for (int j = jMin; j <= jMax; j++) {
				myCameraHandler.setX(j);
				myCameraHandler.firstDerivative(true, (byte) op2);
			}
		}
	}

	/*
	 * This method finds edges using Marr's algorithm. It convolves the image
	 * with a Laplacian of Gaussian kernel. It then uses markZeroCrossings to
	 * find the edges. op2 is used as the threshold for markZeroCrossings. if
	 * op2 is set to 0 then markZeroCrossings isn't called, then the result of
	 * the convolution is written to the output.
	 */
	private void marrEdge() {
		int i, iMin, iMax, j, jMin, jMax;
		byte[] dimensions;

		dimensions = myCameraHandler
				.setConvolutionMatrix(CameraHandler.LOG_5X5_B);

		iMin = (int) (dimensions[1] / 2);
		iMax = myCameraHandler.getCamera().getFrameSize() - dimensions[1]
				+ iMin;

		jMin = (int) (dimensions[0] / 2);
		jMax = myCameraHandler.getCamera().getLineSize() - dimensions[0] + jMin;

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);

		if (op2 > 0) {
			myCameraHandler
					.setDestinationBuffer(CameraHandler.BUFFERID_CUSTOM1);
		} else {
			myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		}

		// loop through the image
		for (i = iMin; i <= iMax; i++) {
			myCameraHandler.setY(i);
			for (j = jMin; j <= jMax; j++) {
				myCameraHandler.setX(j);
				myCameraHandler.convolve(false, true);
			}
		}

		if (op2 > 0) {
			myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_CUSTOM1);
			myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);

			// find zero crossings
			for (i = iMin; i <= iMax; i++) {
				myCameraHandler.setY(i);
				for (j = jMin; j <= jMax; j++) {
					myCameraHandler.setX(j);
					myCameraHandler.markZeroCrossing(true, op2);
				}
			}
		}
	}

	/*
	 * This method detects edges in the image by using the Canny edge detector
	 * algorithm. First the image is blurred, by calling gaussianBlur(). Second
	 * gaussWW() in CameraHandler is called for the entire image. Then
	 * markZeroCrossings is called on the result. op2 is passed as a threshold.
	 * if op2 is 0 then the result of gaussWW (fww) is outputted.
	 */
	private void cannyEdge() {
		int i, iMin, iMax, j, jMin, jMax;
		byte[] dim1, dim2;

		// blur first
		gaussianBlur();

		dim1 = myCameraHandler.setConvolutionMatrix(CameraHandler.GAUSS_5X1_X);
		dim2 = myCameraHandler.setConvolutionMatrix(CameraHandler.GAUSS_1X5_Y);

		iMin = (int) (dim2[1] / 2);
		iMax = myCameraHandler.getCamera().getFrameSize() - dim2[1] + iMin;

		jMin = (int) (dim1[0] / 2);
		jMax = myCameraHandler.getCamera().getLineSize() - dim1[0] + jMin;

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_OUT);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_CUSTOM1);

		for (i = iMin; i <= iMax; i++) {
			myCameraHandler.setY(i);
			for (j = jMin; j <= jMax; j++) {
				myCameraHandler.setX(j);
				myCameraHandler.gaussWW(true);
			}
		}
		if (op2 == 0) {
			myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_CUSTOM1);
			myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
			myCameraHandler.bufferCopy();
		} else {
			myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_CUSTOM1);
			myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);

			// find zero crossings
			for (i = iMin; i <= iMax; i++) {
				myCameraHandler.setY(i);
				for (j = jMin; j <= jMax; j++) {
					myCameraHandler.setX(j);
					myCameraHandler.markZeroCrossing(true, op2);
				}
			}
		}
	}

	/*
	 * This method blurs the image by convolving first with a gaussian kernel in
	 * the X direction and then in the Y direction.
	 */
	private void gaussianBlur() {
		byte[] dimensions;

		// X direction
		dimensions = myCameraHandler
				.setConvolutionMatrix(CameraHandler.GAUSS_5X1);

		int iMin = (int) (dimensions[1] / 2);
		int iMax = myCameraHandler.getCamera().getFrameSize() - dimensions[1]
				+ iMin;

		int jMin = (int) (dimensions[0] / 2);
		int jMax = myCameraHandler.getCamera().getLineSize() - dimensions[0]
				+ jMin;

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_CUSTOM1);

		for (int i = iMin; i <= iMax; i++) {
			myCameraHandler.setY(i);
			for (int j = jMin; j <= jMax; j++) {
				myCameraHandler.setX(j);
				myCameraHandler.convolve(false, true);
			}
		}

		// Y direction
		dimensions = myCameraHandler
				.setConvolutionMatrix(CameraHandler.GAUSS_1X5);

		iMin = (int) (dimensions[1] / 2);
		iMax = myCameraHandler.getCamera().getFrameSize() - dimensions[1]
				+ iMin;

		jMin = (int) (dimensions[0] / 2);
		jMax = myCameraHandler.getCamera().getLineSize() - dimensions[0] + jMin;

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);

		for (int i = iMin; i <= iMax; i++) {
			myCameraHandler.setY(i);
			for (int j = jMin; j <= jMax; j++) {
				myCameraHandler.setX(j);
				myCameraHandler.convolve(false, true);
			}
		}
	}

	/*
	 * These methods can be used to calculated the values needed for different
	 * Gaussian kernels. They are all for sigma = 1
	 *  
	 */
	private double gaussXY(int x, int y) {
		return (2 * y) * -x * Math.pow(Math.E, -((x * x + y * y) / 2.0));
	}

	private double gaussXX(int x) {
		return (x * x - 1) * Math.pow(Math.E, -(x * x / 2.0));
	}

	private double gaussX(int x) {
		return -x * Math.pow(Math.E, -(x * x / 2.0));
	}

	private double gauss(int x, int y) {
		return Math.pow(Math.E, -((x * x + y * y) / 2.0));
	}

	/*
	 * MouseMotion takes the current mouse position and draws a cross in the
	 * picture This is then used to calculate the vector relative to the center
	 * of the picture
	 */
	private void mouseTrack() {
		int yMax = frameSize / 2;
		int xMax = lineSize / 2;
		int X = (int) myCameraHandler.getPosX();
		int Y = (int) myCameraHandler.getPosY();
		int x = 0;
		int y = 0;
		byte[] red = new byte[] { (byte) 255, 0, 0 };
		byte[] green = new byte[] { 0, (byte) 255, 0 };
		byte[] blue = new byte[] { 0, 0, (byte) 255 };
		byte[] white = new byte[] { 0, 0, 0 };

		if (X < 10)
			X = 10;
		if (X >= (xMax * 2 - 10))
			X = xMax * 2 - 10;
		if (Y < 10)
			Y = 10;
		if (Y >= (yMax * 2 - 10))
			Y = yMax * 2 - 10;

		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		myCameraHandler.bufferCopy();
		// Draw the center cross
		for (y = yMax - 10; y < (yMax + 10); y++) {
			if (y == yMax) {
				for (x = xMax - 10; x < (xMax + 10); x++) {
					myCameraHandler.setBufferRGB(x, y, blue);
				}
			} else
				myCameraHandler.setBufferRGB(xMax, y, blue);
		}
		// Draw the cursor cross
		for (y = Y - 10; y < (Y + 10); y++) {
			if (y == Y) {
				for (x = X - 10; x < (X + 10); x++) {
					myCameraHandler.setBufferRGB(x, y, red);
				}
			} else
				myCameraHandler.setBufferRGB(X, y, red);
		}
	}

	private void clearDomColor() {
		R = 0;
		G = 0;
		B = 0;
	}

	private byte[] getDomColor() {
		byte[] RGB = new byte[] { 0, 0, 0 };
		if (R > G)
			if (R > B)
				RGB[0] = R;
			else
				RGB[2] = B;
		else if (G > B)
			RGB[1] = G;
		else
			RGB[2] = B;
		return RGB;
	}

	private void setDomColor(byte[] RGB) {
		if (RGB[0] > R)
			R = RGB[0];
		if (RGB[1] > G)
			G = RGB[1];
		if (RGB[2] > B)
			B = RGB[2];
	}

	public CameraHandler getCameraHandler() {
		return myCameraHandler;
	}
	// <--------- function implemented in 2005 ------------------------------------------------------------->
	/*
	 * useColorModel: uses 'colorModel' with 'threshold'
	 * 
	 * nRGB:	0
	 * c1c2c3:	1
	 * l1l2l3:	2
	 * HSI:		3
	 * default: nRGB
	 * 
	 * RGB is -1 without a threshold
	 */
	private void useColorModel(int colorModel,int threshold) {
		//System.out.println("Color model used: " + colorModel);
		switch (colorModel) {
			case 0:
				// nRGB
				myCameraHandler.normRGB(threshold);
				break;
			case 1:
				// c1c2c3
				myCameraHandler.c1c2c3(threshold);
				break;
			case 2:
				// l1l2l3
				myCameraHandler.l1l2l3(threshold);
				break;
			case 3:
				// HSI
				myCameraHandler.HSI(threshold);
				break;
			default:
				// nRGB
				myCameraHandler.normRGB(threshold);
				break;
		}
	}
	/*
	 * colorModel: loops through the frame and calculates the appropriate colorModel
	 * 			   for the pixels in the frame
	 */
	private void colorModel(int colorModel) {
		if (colorModel!=-1) {
			myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
			myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);		
			
			int I;
			int J;
			
			for (I= 0; I < frameSize; I++)
			{
				for (J= 0; J < lineSize; J++)
				{
					myCameraHandler.setBuffer(J,I);
					useColorModel(colorModel, colorModelThreshold);
				}
			}
			
			myCameraHandler.bufferCopy();
		}
	}
	/*
	 * subtractBackground: loops through the frame and subtract for each
	 * 					   pixel the average value of the background
	 * 					   ( which must be initialized)
	 * 
	 */
	private void subtractBackground()
	{
		int i,iMin,iMax,j,jMin,jMax;
		int blockSize = 1;
		int stepSize = 1;
				
		iMin = Math.round(blockSize/2);
		iMax = myCameraHandler.getCamera().getFrameSize() - Math.round(blockSize/2);
	
		jMin = iMin;
		jMax = myCameraHandler.getCamera().getLineSize() - Math.round(blockSize/2);
	
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		
		for (i=iMin; i<iMax; i=i+stepSize)
		{
			myCameraHandler.setY(i);	
			for (j=jMin; j<jMax; j=j+stepSize)
			{
				myCameraHandler.setBuffer(j,i);
				myCameraHandler.setX(j);
				//useColorModel(currentColorModel, colorModelThreshold); // in case of an other color model
				// backgroundSubstractionThreshold defines when a pixel is 
				// considered as the foreground or the background
				//useColorModel(0,20);
				myCameraHandler.subtract4(blockSize, backgroundSubtractionThreshold); 	
				
			}
		}
		
		
		
	}
	/* 
	 * motionSensor: counts the number of people for the currently initialized doorway class
	 */
	private void motionSensor()
	{
		int stepSize = 1;
		
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setScratchBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		myCameraHandler.bufferCopy();		
		
		doorway.count();
		/*
		Vector motions = new Vector();
		motionSensor1.sense();
		motionSensor1.nextFrame();
		motions = motionSensor1.getMotions();
		
		for (int i=0;i<motions.size();i++) {
			motion m = (motion)motions.elementAt(i);
			m.show();
			
		}
		*/
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.bufferCopy();		
		
		
	}
	/*
	 * boundingBox: detects the motions in two succesive frames after background subtraction
	 * 				(closing\opening), draws bounding boxes around the found regions and draws
	 * 				lines from the center of the regions in a particular direction to indicate the 
	 * 				average speed of the moving region (length of line depends on speed)
	 */
	private void boundingBox() {
		motions.findBoundingBoxes();
	}
	/*
	 * gridSensor:	implementation to detect movements in the horizon of the camera using horizontal
	 * 				and vertical sensors, implementation showed that it only detect global movements
	 *				(strongly depending on the thresholds and size of the placed sensors), so no 
	 *				exact location of movements can be concluded
	 */
	private void gridSensor() {
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setScratchBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		myCameraHandler.bufferCopy();
		grid1.sense();
		grid2.sense();
		Vector motions1 = grid1.getHorizontalMotions();
		Vector motions2 = grid2.getVerticalMotions();
		for (int i=0;i<motions1.size();i++) {
			if (i==0) {
				System.out.println("<--------- Horizontal Motions --------------->");
			}
			motion temp = (motion) motions1.elementAt(i);
			temp.show();
			
		}
		for (int i=0;i<motions2.size();i++) {
			if (i==0) {
				System.out.println("<--------- Vertical Motions --------------->");
			}
			motion temp = (motion) motions2.elementAt(i);
			temp.show();
			
		}
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_IN);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.bufferCopy();	
		
	}
	/*
	 * close: 	after background subtraction, closing removes the noise in the image
	 * 			depending on the size of the stamp (n) used
	 */
	private void close(int n)
	{
		//myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		int stepSize = 1;
		int i,iMin,iMax,j,jMin,jMax;
		
		iMin = Math.round(1/2);
		iMax = myCameraHandler.getCamera().getFrameSize() - Math.round(1/2);
		jMin = iMin;
		jMax = myCameraHandler.getCamera().getLineSize() - Math.round(1/2);
		
		// loop de image door. (0,0) ligt in de linker onderhoek		
		for (i=iMin; i<iMax; i=i+stepSize)
		{
			myCameraHandler.setY(i);
			for (j=jMin; j<jMax; j=j+stepSize)
			{				
				myCameraHandler.setX(j);
				myCameraHandler.close(n); 	
			}
		}
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		myCameraHandler.bufferCopy();
	}
	/*
	 * open:	after background subtraction and closing, opening enlarges the
	 * 			white pixels in the image, which could improve the performance
	 * 			of the motion detection with regions
	 */
	private void open(int n)
	{
		//myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		int stepSize = 1;
		int i,iMin,iMax,j,jMin,jMax;
		int blockSize = 1;
		
		iMin = Math.round(blockSize/2);
		iMax = myCameraHandler.getCamera().getFrameSize() - Math.round(blockSize/2);
		jMin = iMin;
		jMax = myCameraHandler.getCamera().getLineSize() - Math.round(blockSize/2);
		
		// after closing
		//myCameraHandler.resetBufferCustom1();
		
		// loop de image door. (0,0) ligt in de linker onderhoek		
		for (i=iMin; i<iMax; i=i+stepSize)
		{
			myCameraHandler.setY(i);
			for (j=jMin; j<jMax; j=j+stepSize)
			{				
				myCameraHandler.setX(j);
				myCameraHandler.open(n); 	
			}
		}
		myCameraHandler.setSourceBuffer(CameraHandler.BUFFERID_CUSTOM1);
		myCameraHandler.setDestinationBuffer(CameraHandler.BUFFERID_OUT);
		myCameraHandler.bufferCopy();
	}
}