package VM2;
import java.util.*;
/*
 * @author Daan van Schuppen (UVA)
 *
 * doorwayCounting: class which is able to count the number of people entering 
 * 					or leaving a room through the doorway by the use of grid sensors 
 * 
 */

public class doorwayCounting
{
	// total number of people in the room
	public int numberOfPeople = 0;
	// grid sensors
	public sensorGrid ms1,ms2;
	public int waitingTime=100;
	// time at which the counting sensor is inactive after counting a person
	public int inactiveTime=15;
	// boolean value defines if the grid sensor are horizontal or vertical 
	// true: horizontal
	// false: vertical
	public boolean horizontal;
	
	public doorwayCounting(CameraHandler myCameraHandler) {
		// default constructor, places two horizontal grid sensors
		this.ms1 = new sensorGrid(myCameraHandler,80,150,100,200,"ms1",1,6,4,1,0.6,true,false, 13);
		this.ms2 = new sensorGrid(myCameraHandler,400,150,100,200,"ms2",1,6,4,1,0.6,true, false, 13);
		this.horizontal = true;
	}
	/* 
	 * myCameraHandler: cameraHandler
	 * positions:		array containing the positions of the to be placed grid sensors
	 * horizontal:		defines if either two horizontal grid sensors are used or one vertical
	 * 					grid sensor
	 * 					true -> horizontal
	 * 					false -> vertical
	 */
	public doorwayCounting(CameraHandler myCameraHandler,int[] positions, boolean horizontal) {
		// constructor, places either two horizontal grid sensors or one vertical grid sensor at 
		// the positions in 'positions' array
		this.horizontal = horizontal;
		if (horizontal) {
			// two horizontal grid sensors
			int w1 = Math.abs(positions[2] - positions[0]);
			int h1 = Math.abs(positions[3] - positions[1]);
			
			int w2 = Math.abs(positions[6] - positions[4]);
			int h2 = Math.abs(positions[7] - positions[5]);
			
			this.ms1 = new sensorGrid(myCameraHandler,positions[0],positions[1]+h1/2,w1,h1,"ms1",1,10,5,3,0.3,true,false, 13);
			this.ms2 = new sensorGrid(myCameraHandler,positions[4],positions[5]+h2/2,w2,h2,"ms2",1,10,5,3,0.3,true, false, 13);
		} else {
			// one vertical grid sensor
			int w1 = Math.abs(positions[2] - positions[0]);
			int h1 = Math.abs(positions[3] - positions[1]);
			
			this.ms1 = new sensorGrid(myCameraHandler,positions[0],positions[1]+h1/2,w1,h1,"ms1",1,10,5,3,0.3,false,true, 13);
		}
	}
	/*
	 * getNumberOfPeople: returns the current counted number of people
	 */
	public int getNumberOfPeople() {
		return this.numberOfPeople;
	}
	/*
	 * setNumberOfPeople: set the number of people to n
	 */
	public void setNumberOfPeople(int n) {
		this.numberOfPeople = n;
	}
	/*
	 * count: checks if the horizontal/vertical grid sensors detected a motion in a 
	 * 		  particular direction in the current frame
	 */
	public void count() {
		if (horizontal) {
			count_horizontal();
		} else {
			count_vertical();
		}
	}
	/*
	 * count: checks if the horizontal grid sensors detected a motion in a either left or 
	 * 		  right direction, according to the grid sensors it increases or decreases
	 * 		  the number of people
	 */
	public void count_horizontal() {
		// two vectors containing the motions of the two grid sensors
		Vector motions1 = new Vector();
		Vector motions2 = new Vector();
		
		boolean left,right;
		left = false;
		right = false;
		// first let the grid sensors sense the current frame
		ms1.sense();
		ms2.sense();
		waitingTime++;
		// if the counter is active
		if (waitingTime > inactiveTime) {
			// get the motions of the grid sensors
			motions1 = ms1.getHorizontalMotions();
			motions2 = ms2.getHorizontalMotions();
			
			// if only the first grid sensor detects motion
			if (motions1.size()>0&&motions2.size()==0) {
				for (int i=0;i<motions1.size();i++) {
					motion m = (motion)motions1.elementAt(i);
					// a left motion is detected
					if (m.getDirection()==0&&!left) {
						this.numberOfPeople++;
						// block multiple left motions
						// otherwise the sensor detects too many people for one person 
						left = true;
						// let the counter be inactive for a particular number of frames
						waitingTime=0;
					} else if (m.getDirection()==1&&!right) {
						// a right motion is detected
						this.numberOfPeople--;
						// block multiple right motions
						// otherwise the sensor detects too many people for one person 
						right = true;
						// let the counter be inactive for a particular number of frames
						waitingTime=0;
					}
					System.out.println(" Number of people --> " + this.numberOfPeople);
					// show the motion
					//m.show();
					
				}
			}
			left = false;
			right = false;
			// if only the second grid sensor detects motion
			if (motions2.size()>0&&motions1.size()==0) {
				for (int i=0;i<motions2.size();i++) {
					motion m = (motion)motions2.elementAt(i);
					// a left motion is detected
					if (m.getDirection()==1&&!left) {
						this.numberOfPeople++;
						// block multiple left motions
						// otherwise the sensor detects too many people for one person 
						left = true;
						// let the counter be inactive for a particular number of frames
						waitingTime=0;
						
						
					} else if (m.getDirection()==0&&!right){
						// a right motion is detected
						this.numberOfPeople--;
						// block multiple right motions
						// otherwise the sensor detects too many people for one person 
						right = true;
						// let the counter be inactive for a particular number of frames
						waitingTime=0;
					}
					System.out.println(" Number of people --> " + this.numberOfPeople);
					// show the motion
					//m.show();
					
				}
			}
		}
	}
	public void count_vertical() {
		// two vectors containing the motions of the two grid sensors
		Vector motions1 = new Vector();
		boolean left,right;
		left = false;
		right = false;
		ms1.sense();
		waitingTime++;
		if (waitingTime > inactiveTime) {
			motions1 = ms1.getVerticalMotions();
			for (int i=0;i<motions1.size();i++) {
				motion m = (motion)motions1.elementAt(i);
				if (m.getDirection()==0&&!left) {
					this.numberOfPeople++;
					left = true;
					waitingTime=0;
				} else if (m.getDirection()==1&&!right) {
					this.numberOfPeople--;
					right = true;
					waitingTime=0;
				}
				System.out.println(" Number of people --> " + this.numberOfPeople);
				m.show();
				
			}
		}
	}

}
