package VM2;
import java.util.*;

/*
 * @author Derk Crezee & Daan van Schuppen (UVA)
 *
 * The SensorGrid class is a class where multiple sensors are combined to
 * determine the direction a general motion in horizontal and/or vertical
 * direction.
 */
public class sensorGrid
{
	private String name;
	private CameraHandler myCameraHandler;
	private int blockSize;
	
	// threshold of the sensors, if exceeded the sensor will be inactive
	// for a certain time
	private double threshold1 = 0.4;
	
	private int frameNumber = 0;
	
	// location of the upper left corner at which the sensor grid starts
	private int xpos;
	private int ypos;
	// width and height of the sensor grid
	private int width;
	private int height;
	
	// horizontal and\or vertical placement of sensors
	private boolean horizontal = true;
	private boolean vertical = true;
	

	private int horizontalMotion = 0;
	private int verticalMotion = 0;
	private int[] horizontalSensorFrames;
	private int[] verticalSensorFrames;
	// TODO comment: threshold
	int frameThreshold = 8;
	private Vector horizontalSensors;
	private Vector verticalSensors;
	private int deactivationTime;
	private int space;
	private int numberOfSensors;
	private int sensorLength;
	private int stepSize;
	private Vector verticalMotions;
	private Vector horizontalMotions;
		
	/*
	 * - myCameraHandler: CameraHandler <br>
	 * - xpos: x position of the top left point of the grid <br>
	 * - ypos: y position of the top left point of the grid <br>
	 * - width: width of grid <br>
	 * - height: height of grid <br>
	 * - name: name of the sensorGrid <br>
	 * - blockSize: <br>
	 * - sensorLength: width of horizontal sensors and height of vertical sensors <br>
	 * - space: number of pixels between sensors <br>
	 * - stepSize: space between active pixel sensors in a sensor <br>
	 * - threshold: threshold for sensors to become active <br>
	 * - horizontal: true if horizontal grid
	 * - vertical: true if vertical grid
	 * - deactivationTime: number of frames at which the sensor is not active after exceeding the threshold
	 */
	public sensorGrid(CameraHandler camHandler, int xpos, int ypos, int width, int height,String name,int blockSize, 
			int sensorLength, int space, int stepSize, double threshold, boolean horizontal, boolean vertical, int deactivationTime) {
		
		this.myCameraHandler = camHandler; 
		this.sensorLength= sensorLength;
		this.xpos = xpos;
		this.ypos = ypos; 
		this.width = width; 
		this.height = height; 
		this.name = name;
		this.blockSize = blockSize;
		this.sensorLength = sensorLength;
		this.space = space; 
		this.stepSize = stepSize;
		this.threshold1 = threshold;
		this.horizontal = horizontal;
		this.vertical = vertical;
		this.deactivationTime = deactivationTime;
		initialize(); 
	}
	
	
	/*
	 * Initializes the sensor grid
	 * 
	 * Places all the horizontal and\or vertical sensors in the grid
	 */
	private void initialize() {	
		// There must be sensors
		if( !(horizontal||vertical) ) {
			System.out.println("No sensors");
			System.exit(0);
		}
		
		frameNumber = 0;		
		
		int numberOfSensors;
		
		// Loop through horizontal sensors
		if(horizontal) {
			numberOfSensors = 0;
			horizontalSensors = new Vector();
			
			for (int i=xpos; i < xpos + width; i = i+sensorLength+space) {
				// Add sensor to grid			
				horizontalSensors.add(new Sensor(myCameraHandler,i,ypos,sensorLength,height,this.name + "_horizontalSensor_" + numberOfSensors,this.blockSize,this.stepSize,deactivationTime,this.threshold1) );
				numberOfSensors++;
			}
			horizontalSensorFrames = new int[numberOfSensors];
		}	
		
		// Loop through vertical sensors
		if(vertical) {
			numberOfSensors = 0;
			verticalSensors = new Vector();
			
			for (int i=ypos; i < ypos + height; i = i+sensorLength+stepSize) {
				// Add sensor to grid
				verticalSensors.add(new Sensor(myCameraHandler,xpos,i,width,sensorLength,this.name + "_verticalSensor_" + numberOfSensors,this.blockSize,this.stepSize,deactivationTime,this.threshold1) );
				numberOfSensors++;
			}
			
			verticalSensorFrames = new int[numberOfSensors];			
		}
	}
	
	
	/*
	 * Sense with all sensors in grid
	 */
	public void sense() {
		frameNumber++;
		Sensor tempSensor;
		double sensorValue;
		
		if(horizontal) {
			for (int i=0;i<horizontalSensors.size();i++) {
				tempSensor = (Sensor)horizontalSensors.elementAt(i);
				// let each sensor sense
				sensorValue = tempSensor.sense();
				// if the sensorValue is greater than the threshold
				// store the frameNumber at the location of the sensor
				// to detect motion
				if(sensorValue > threshold1) {					
					horizontalSensorFrames[i] = frameNumber;
				}
			}
		}
		
		if(vertical) {
			for (int i=0;i<verticalSensors.size();i++) {
				tempSensor = (Sensor)verticalSensors.elementAt(i);
				// let each sensor sense
				sensorValue = tempSensor.sense();
				// if the sensorValue is greater than the threshold
				// store the frameNumber at the location of the sensor
				// to detect motion
				if(sensorValue > threshold1) {					
					verticalSensorFrames[i] = frameNumber;
				}
			}			
		}
		
		// detect motion of the horizontal and vertical sensorGrid
		if(horizontal) {
			this.horizontalMotions = determineMotion(horizontalSensorFrames);
		}
		if(vertical) {
			this.verticalMotions = determineMotion(verticalSensorFrames);
		}
	}
	/*
	 * getNumberOfSensors: returns the number of sensors placed in the grid
	 */
	public int getNumberOfSensors() {
		return this.numberOfSensors;
	}
	/* 
	 * getVerticalMotions: returns the vertical motions of the current frame
	 */
	public Vector getVerticalMotions() {
		return this.verticalMotions;
	}
	/*
	 * getHorizontalMotions: returns the horizontal motions of the current frame
	 */
	public Vector getHorizontalMotions() {
		return this.horizontalMotions;
	}
	
	
	
	/*
	 * determineMotion: determines the horizontal or vertical motions according
	 * 					to which motionFrame is called with
	 */
	public Vector determineMotion(int[] motionFrame) {
		boolean motionFound = false;
		int motion, current,difference,start, last;
		
		// to store visited frame numbers
		Hashtable frames = new Hashtable();
		last = 0;
		
		// contains the motions
		Vector motions = new Vector();
		
		// determine the motion from left to right if horizontal
		// or from up to down if vertical sensor grid is used
		//
		// a motion is detected when the motion frame (array containing the frame numbers at which the 
		// sensors were active most recently) contain an increasing or decreasing (direction of motion)
		// trace of frame numbers
		
		// motion from left to right: 
		// loop trhough the motion frame and check for each frame number at sensor i if there is
		// a motion trace
		for (int i=0;i<motionFrame.length;i++) {
			// sort of speed measure
			motion = 0;
			
			// framenumber of sensor i
			current = motionFrame[i];
			start = motionFrame[i]; // original index
			
			// check for sensor i if it is a frame number already visited
			// and if not check if there is a motion trace
			if (frames.get(new Integer(start))==null) {
				frames.put(new Integer(start),new Integer(1));
				
				// motion trace
				for (int j=i;j<motionFrame.length;j++) {
					difference = motionFrame[j] - current;
					
					// a motion can only occur recently 
					if (difference < frameThreshold && difference>0) {
						motion = motion + difference; 
						frames.put(new Integer(motionFrame[j]),new Integer(1));
						current = difference;
						last = j;
					}
				}
			}
			// there is a motion, add motion
			if (motion > 0) {
				motions.add(new motion(1,i,last, this.name,motion));
				motionFound = true;
			}
			motion = 0;
		}
		frames = new Hashtable();
		// motions from right to left:
		for (int i=motionFrame.length-1;i>=0;i--) {
			motion = 0;
			current = motionFrame[i];
			start = motionFrame[i];
			if (frames.get(new Integer(start))==null) {
				frames.put(new Integer(start),new Integer(1));
				for (int j=i;j>=0;j--) {
					difference = motionFrame[j] - current;
					if (difference < frameThreshold && difference>0) {
						motion = motion + difference; 
						frames.put(new Integer(motionFrame[j]),new Integer(1));
						current = difference;
						last = j;
					}
				}
			}
			if (motion>0) {
				motions.add(new motion(0,i,last, this.name,motion));
				motionFound = true;
			}
			motion = 0;
		}
		// if there were motions found, reset the motion frame to detect new motions 
		// in the next frames
		if (motionFound) {
			for (int i=0;i<motionFrame.length;i++) {
				motionFrame[i] = 0;
				
			}
		}
		return motions;
	}
	
}